# -*- coding: utf-8 -*-
"""
Created on Tue Oct 26 12:46:55 2021

@author: jpeacock
"""
# =============================================================================
# Imports
# =============================================================================
import unittest
from pathlib import Path
import pandas as pd

from mth5.clients.make_mth5 import MakeMTH5
from obspy.clients.fdsn.header import FDSNNoDataException
from mth5.utils.mth5_logger import setup_logger

# =============================================================================
# Test various inputs for getting metadata
# =============================================================================


@unittest.skipIf(
    "peacock" not in str(Path(__file__).as_posix()),
    "Downloading from IRIS takes too long",
)
class TestMakeMTH5(unittest.TestCase):
    """
    test a csv input to get metadata from IRIS

    """

    @classmethod
    def setUpClass(self):

        self.make_mth5 = MakeMTH5(mth5_version="0.2.0")
        self.make_mth5.client = "IRIS"

        channels = ["LFE", "LFN", "LFZ", "LQE", "LQN"]
        CAS04 = ["8P", "CAS04", "2020-06-02T18:00:00", "2020-07-13T19:00:00"]
        NVR08 = ["8P", "NVR08", "2020-06-02T18:00:00", "2020-07-13T19:00:00"]

        request_list = []
        for entry in [CAS04, NVR08]:
            for channel in channels:
                request_list.append(
                    [entry[0], entry[1], "", channel, entry[2], entry[3]]
                )
        self.logger = setup_logger("test_make_mth5_v2")
        self.csv_fn = Path().cwd().joinpath("test_inventory.csv")
        self.mth5_path = Path().cwd()

        self.stations = ["CAS04", "NVR08"]
        self.channels = ["LQE", "LQN", "LFE", "LFN", "LFZ"]

        # Turn list into dataframe
        self.metadata_df = pd.DataFrame(
            request_list, columns=self.make_mth5.column_names
        )
        self.metadata_df.to_csv(self.csv_fn, index=False)

        self.metadata_df_fail = pd.DataFrame(
            request_list,
            columns=["net", "sta", "loc", "chn", "startdate", "enddate"],
        )

    def test_df_input_inventory(self):
        inv, streams = self.make_mth5.get_inventory_from_df(
            self.metadata_df, data=False
        )
        with self.subTest(name="stations"):
            self.assertListEqual(
                sorted(self.stations),
                sorted([ss.code for ss in inv.networks[0].stations]),
            )
        with self.subTest(name="channels_CAS04"):
            self.assertListEqual(
                sorted(self.channels),
                sorted(
                    list(
                        set(
                            [
                                ss.code
                                for ss in inv.networks[0].stations[0].channels
                            ]
                        )
                    )
                ),
            )
        with self.subTest(name="channels_NVR08"):
            self.assertListEqual(
                sorted(self.channels),
                sorted(
                    list(
                        set(
                            [
                                ss.code
                                for ss in inv.networks[0].stations[1].channels
                            ]
                        )
                    )
                ),
            )

    def test_csv_input_inventory(self):
        inv, streams = self.make_mth5.get_inventory_from_df(
            self.csv_fn, data=False
        )
        with self.subTest(name="stations"):
            self.assertListEqual(
                sorted(self.stations),
                sorted([ss.code for ss in inv.networks[0].stations]),
            )

        with self.subTest(name="channels_CAS04"):
            self.assertListEqual(
                sorted(self.channels),
                sorted(
                    list(
                        set(
                            [
                                ss.code
                                for ss in inv.networks[0].stations[0].channels
                            ]
                        )
                    )
                ),
            )
        with self.subTest(name="channels_NVR08"):
            self.assertListEqual(
                sorted(self.channels),
                sorted(
                    list(
                        set(
                            [
                                ss.code
                                for ss in inv.networks[0].stations[1].channels
                            ]
                        )
                    )
                ),
            )

    def test_fail_csv_inventory(self):
        self.assertRaises(
            ValueError,
            self.make_mth5.get_inventory_from_df,
            *(self.metadata_df_fail, self.make_mth5.client, False),
        )

    def test_fail_wrong_input_type(self):
        self.assertRaises(
            ValueError,
            self.make_mth5.get_inventory_from_df,
            *(("bad tuple", "bad_tuple"), self.make_mth5.client, False),
        )

    def test_fail_non_existing_file(self):
        self.assertRaises(
            IOError,
            self.make_mth5.get_inventory_from_df,
            *("c:\bad\file\name", self.make_mth5.client, False),
        )

    @unittest.skipIf(
        "peacock" not in str(Path().cwd().as_posix()),
        "Test is too long, have to download data from IRIS",
    )
    def test_make_mth5(self):
        try:
            m = self.make_mth5.make_mth5_from_fdsnclient(
                self.metadata_df, self.mth5_path, interact=True
            )

            sg = m.get_survey("CONUS_South")
            with self.subTest(name="stations"):
                self.assertListEqual(
                    self.stations, sg.stations_group.groups_list
                )
            with self.subTest(name="CAS04_runs"):
                self.assertListEqual(
                    ["Transfer_Functions", "a", "b", "c", "d"],
                    m.get_station("CAS04", "CONUS_South").groups_list,
                )
            for run in ["a", "b", "c", "d"]:
                for ch in ["ex", "ey", "hx", "hy", "hz"]:
                    with self.subTest(name=f"has data CAS04.{run}.{ch}"):
                        x = m.get_channel("CAS04", run, ch, "CONUS_South")
                        self.assertTrue(abs(x.hdf5_dataset[()].mean()) > 0)
            for run in ["a", "b", "c"]:
                for ch in ["ex", "ey", "hx", "hy", "hz"]:
                    x = m.get_channel("NVR08", run, ch, "CONUS_South")
                    with self.subTest(name=f"has data NVR08.{run}.{ch}"):
                        self.assertTrue(abs(x.hdf5_dataset[()].mean()) > 0)

                    with self.subTest(name="filters"):
                        self.assertTrue(x.metadata.filter.name != [])

            m.close_mth5()
            m.filename.unlink()
        except FDSNNoDataException as error:
            self.logger.warning(
                "The requested data could not be found on the FDSN IRIS server, check data availability"
            )
            self.logger.error(error)

            raise Exception(
                "The requested data could not be found on the FDSN IRIS server, check data availability"
            )

    @classmethod
    def tearDownClass(self):
        self.csv_fn.unlink()


# =============================================================================
# Run
# =============================================================================
if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
This is a utility function to get the appropriate reader for a given file type and 
return the appropriate object of :class:`mth5.timeseries`

This setup to be like plugins but a hack cause I did not find the time to set
this up properly as a true plugin.  

If you are writing your own reader you need the following structure:
    
    * Class object that will read the given file
    * a reader function that is read_{file_type}, for instance read_nims
    * the return value is a :class:`mth5.timeseries.MTTS` or 
      :class:`mth5.timeseries.RunTS` object and any extra metadata in the form
      of a dictionary with keys as {level.attribute}.
    
.. code-block:: python
    
    class NewFile
        def __init__(self, fn):
            self.fn = fn
        
        def read_header(self):
            return header_information
        
        def read_newfile(self):
            ex, ey, hx, hy, hz = read_in_channels_as_MTTS
            return RunTS([ex, ey, hx, hy, hz])
        
    def read_newfile(fn):
        new_file_obj = NewFile(fn)
        run_obj = new_file_obj.read_newfile()
        
        return run_obj, extra_metadata
        
Then add your reader to the reader dictionary so that those files can be read.
        
.. seealso:: Existing readers for some guidance found in `mth5.io`

Created on Wed Aug 26 10:32:45 2020

:author: Jared Peacock

:license: MIT

"""
# =============================================================================
# Imports
# =============================================================================

from pathlib import Path
import logging

from mth5.io import zen, nims, usgs_ascii, miniseed, lemi, phoenix

logger = logging.getLogger(__name__)
# =============================================================================
# generic reader for any file type
# =============================================================================
readers = {
    "zen": {"file_types": ["z3d"], "reader": zen.read_z3d},
    "nims": {"file_types": ["bin", "bnn"], "reader": nims.read_nims},
    "usgs_ascii": {
        "file_types": ["asc", "zip"],
        "reader": usgs_ascii.read_ascii,
    },
    "miniseed": {
        "file_types": ["miniseed", "ms", "mseed"],
        "reader": miniseed.read_miniseed,
    },
    "lemi424": {
        "file_types": ["txt"],
        "reader": lemi.read_lemi424,
    },
    "phoenix": {
        "file_types": ["bin", "td_150", "td_24k"],
        "reader": phoenix.read_phoenix,
    },
}


def get_reader(extension):
    """

    get the proper reader for file extension

    :param extension: file extension
    :type extension: string
    :return: the correct function to read the file
    :rtype: function

    """
    if extension in ["bin"]:
        logger.warning(
            "Suggest inputing file type, bin could be nims or phoenix"
        )
    for key, vdict in readers.items():
        if extension.lower() in vdict["file_types"]:
            return key, vdict["reader"]
    msg = f"Could not find a reader for file type {extension}"
    logger.error(msg)
    raise ValueError(msg)


def read_file(fn, file_type=None, **kwargs):
    """
    This is the universal reader for MT time series.  This will pick out the
    proper reader given the file type or extension.  Keyworkd arguments will
    depend on the reader and file type.

    :param fn: full path to file
    :type fn: string or :class:`pathlib.Path`
    :param string file_type: a specific file time if the extension is ambiguous.
    :return: channel or run time series object
    :rtype: :class:`mth5.timeseries.MTTS` or :class:`mth5.timeseries.RunTS`

    """

    if isinstance(fn, (list, tuple)):
        fn = [Path(ff) for ff in fn]
        if not fn[0].exists():
            msg = f"Could not find file {fn}. Check path."
            logger.error(msg)
            raise IOError(msg)
        file_ext = fn[0].suffix[1:]

    else:
        fn = Path(fn)
        if not fn.exists():
            msg = f"Could not find file {fn}. Check path."
            logger.error(msg)
            raise IOError(msg)
        file_ext = fn.suffix[1:]

    if file_type is not None:
        try:
            file_reader = readers[file_type]["reader"]
        except KeyError:
            msg = (
                f"Reader for the file type {file_type} does not currently exist. "
                f"Current readers {list(readers.keys())}"
            )
            logger.error(msg)
            raise KeyError(msg)
    else:
        file_type, file_reader = get_reader(file_ext)
    return file_reader(fn, **kwargs)



# -*- coding: utf-8 -*-
"""
Created on Mon Sep 19 14:56:09 2022

@author: jpeacock
"""

# =============================================================================
# Imports
# =============================================================================
import unittest
from pathlib import Path
from mth5.utils.helpers import initialize_mth5
from mth5.helpers import close_open_files
from mth5.mth5 import MTH5

# =============================================================================


class TestInitializeMTH5(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.wd = Path().cwd()
        self.m = MTH5()
        self.m.open_mth5(self.wd.joinpath("test.h5"), mode="a")

    def test_has_open_file(self):
        self.m = initialize_mth5(self.wd.joinpath("test.h5"), "w")
        self.assertIsInstance(self.m, MTH5)

    @classmethod
    def tearDownClass(self):
        close_open_files()
        self.wd.joinpath("test.h5").unlink()


class TestInitializeMTH502(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.wd = Path().cwd()
        self.m = MTH5()
        self.m.open_mth5(self.wd.joinpath("test.h5"), mode="a")

    def test_has_open_file(self):
        m = initialize_mth5(self.wd.joinpath("test.h5"), "a")
        self.assertIsInstance(m, MTH5)

    @classmethod
    def tearDownClass(self):
        self.m.close_mth5()
        self.wd.joinpath("test.h5").unlink()


# =============================================================================
# Run
# =============================================================================
if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Created on Wed Mar 23 14:09:38 2022

@author: jpeacock
"""

# =============================================================================
# Imports
# =============================================================================
import pandas as pd
import numpy as np
import h5py

from mth5 import CHANNEL_DTYPE
from mth5.tables import MTH5Table

# =============================================================================


class ChannelSummaryTable(MTH5Table):
    """
    Object to hold the channel summary and provide some convenience functions
    like fill, to_dataframe ...
    
    """

    def __init__(self, hdf5_dataset):
        super().__init__(hdf5_dataset)
        self._dtype = CHANNEL_DTYPE

    def to_dataframe(self):
        """
        Create a pandas DataFrame from the table for easier querying.
        
        :return: Channel Summary 
        :rtype: :class:`pandas.DataFrame`

        """

        df = pd.DataFrame(self.array[()])
        for key in [
            "survey",
            "station",
            "run",
            "component",
            "measurement_type",
            "units",
        ]:
            setattr(df, key, getattr(df, key).str.decode("utf-8"))
        df.start = pd.to_datetime(df.start.str.decode("utf-8"))
        df.end = pd.to_datetime(df.end.str.decode("utf-8"))

        return df

    def summarize(self):
        """
        
        :return: DESCRIPTION
        :rtype: TYPE

        """

        self.clear_table()

        def recursive_get_channel_entry(group):
            """
            a function to get channel entry
            """
            if isinstance(group, (h5py._hl.group.Group, h5py._hl.files.File)):
                for key, node in group.items():
                    recursive_get_channel_entry(node)
            elif isinstance(group, h5py._hl.dataset.Dataset):
                try:
                    ch_type = group.attrs["type"]
                    if ch_type in ["electric", "magnetic", "auxiliary"]:
                        ch_entry = np.array(
                            [
                                (
                                    group.parent.parent.parent.parent.attrs["id"],
                                    group.parent.parent.attrs["id"],
                                    group.parent.attrs["id"],
                                    group.parent.parent.attrs["location.latitude"],
                                    group.parent.parent.attrs["location.longitude"],
                                    group.parent.parent.attrs["location.elevation"],
                                    group.attrs["component"],
                                    group.attrs["time_period.start"],
                                    group.attrs["time_period.end"],
                                    group.size,
                                    group.attrs["sample_rate"],
                                    group.attrs["type"],
                                    group.attrs["measurement_azimuth"],
                                    group.attrs["measurement_tilt"],
                                    group.attrs["units"],
                                    group.ref,
                                    group.parent.ref,
                                    group.parent.parent.ref,
                                )
                            ],
                            dtype=CHANNEL_DTYPE,
                        )
                        self.add_row(ch_entry)

                except KeyError:
                    pass

        recursive_get_channel_entry(self.array.parent)



# -*- coding: utf-8 -*-
"""
Created on Fri May  6 12:39:34 2022

@author: jpeacock
"""

# =============================================================================
# Imports
# =============================================================================
from pathlib import Path
from .readers import (
    NativeReader,
    DecimatedContinuousReader,
    DecimatedSegmentedReader,
)

from mth5.timeseries import ChannelTS

# =============================================================================
READERS = {
    "bin": NativeReader,
    "td_24k": DecimatedSegmentedReader,
    "td_150": DecimatedContinuousReader,
    "td_30": DecimatedContinuousReader,
}


def open_phoenix(file_name, **kwargs):
    """
    Will put the file into the appropriate container


    :param file_name: full path to file to open
    :type file_name: string or :class:`pathlib.Path`
    :return: The appropriate container based on file extension
    :rtype:

    """
    file_name = Path(file_name)
    extension = file_name.suffix[1:]

    # need to put the data into a TS object

    return READERS[extension](file_name, **kwargs)


def read_phoenix(file_name, **kwargs):
    """
    Read a Phoenix file into a ChannelTS object
    :param file_name: DESCRIPTION
    :type file_name: TYPE
    :return: DESCRIPTION
    :rtype: TYPE

    """

    phnx_obj = open_phoenix(file_name, **kwargs)

    return phnx_obj.to_channel_ts()



# -*- coding: utf-8 -*-
"""
Created on Wed Jun  9 08:58:15 2021

:copyright: 
    Jared Peacock (jpeacock@usgs.gov)

:license: MIT

"""

# =============================================================================
# Imports
# =============================================================================
from mt_metadata.timeseries.filters import CoefficientFilter

from mth5.groups.base import BaseGroup

# =============================================================================
#  COEFFCIENT Group
# =============================================================================


class CoefficientGroup(BaseGroup):
    """
    Container for Coefficient type filters
    """

    def __init__(self, group, **kwargs):
        super().__init__(group, **kwargs)

    @property
    def filter_dict(self):
        """

        Dictionary of available coefficient filters

        :return: DESCRIPTION
        :rtype: TYPE
        """
        f_dict = {}
        for key in self.hdf5_group.keys():
            coefficient_group = self.hdf5_group[key]
            f_dict[key] = {
                "type": coefficient_group.attrs["type"],
                "hdf5_ref": coefficient_group.ref,
            }

        return f_dict

    def add_filter(self, name, coefficient_metadata):
        """
        Add a coefficient Filter

        :param name: DESCRIPTION
        :type name: TYPE
        :param coefficient_metadata: DESCRIPTION
        :type coefficient_metadata: TYPE
        :return: DESCRIPTION
        :rtype: TYPE

        """
        # create a group for the filter by the name
        coefficient_filter_group = self.hdf5_group.create_group(name)

        # fill in the metadata
        coefficient_filter_group.attrs.update(coefficient_metadata)

        return coefficient_filter_group

    def remove_filter(self):
        pass

    def get_filter(self, name):
        """
        Get a filter from the name

        :param name: name of the filter
        :type name: string

        :return: HDF5 group of the ZPK filter
        """
        return self.hdf5_group[name]

    def from_object(self, coefficient_object):
        """
        make a filter from a :class:`mt_metadata.timeseries.filters.CoefficientFilter`

        :param zpk_object: MT metadata Coefficient Filter
        :type zpk_object: :class:`mt_metadata.timeseries.filters.CoefficientFilter`

        """

        if not isinstance(coefficient_object, CoefficientFilter):
            msg = f"Filter must be a CoefficientFilter not {type(coefficient_object)}"
            self.logger.error(msg)
            raise TypeError(msg)

        input_dict = coefficient_object.to_dict(single=True, required=False)
        for k, v in input_dict.items():
            if v is None:
                input_dict[k] = str(v)
        coefficient_group = self.add_filter(coefficient_object.name, input_dict)
        return coefficient_group

    def to_object(self, name):
        """
        make a :class:`mt_metadata.timeseries.filters.CoefficientFilter` object

        :return: DESCRIPTION
        :rtype: TYPE

        """

        coefficient_group = self.get_filter(name)

        coefficient_obj = CoefficientFilter(**coefficient_group.attrs)

        return coefficient_obj



# -*- coding: utf-8 -*-
"""
Created on Thu Sep 15 15:47:42 2022

@author: jpeacock
"""
# =============================================================================
# Imports
# =============================================================================
import unittest
from pathlib import Path
from collections import OrderedDict
import numpy as np

from mth5.io.usgs_ascii import AsciiMetadata

# =============================================================================


@unittest.skipIf("peacock" not in str(Path(__file__).as_posix()), "local file")
class TestAsciiMetadata(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.header = AsciiMetadata(
            fn=r"C:\Users\jpeacock\OneDrive - DOI\mt\usgs_ascii\rgr003a_converted.asc"
        )
        self.header.read_metadata()
        self.maxDiff = None

    def test_fn(self):
        self.assertIsInstance(self.header.fn, Path)

    def test_coordinate_system(self):
        self.assertEqual(self.header.coordinate_system, "geographic north")

    def test_elevation(self):
        self.assertTrue(np.isclose(self.header.elevation, 1803.00, 0))

    def test_missing_data_flag(self):
        self.assertEqual(self.header.missing_data_flag, "1.000e+09")

    def test_end(self):
        self.assertEqual(self.header.end, "2012-08-24T16:25:26+00:00")

    def test_ex(self):
        self.assertDictEqual(
            self.header.ex_metadata.to_dict(single=True),
            OrderedDict(
                [
                    ("channel_number", 32),
                    ("component", "ex"),
                    ("data_quality.rating.value", 0),
                    ("dipole_length", 100.0),
                    ("filter.applied", [False]),
                    ("filter.name", []),
                    ("measurement_azimuth", 9.0),
                    ("measurement_tilt", 0.0),
                    ("negative.elevation", 0.0),
                    ("negative.id", None),
                    ("negative.latitude", 0.0),
                    ("negative.longitude", 0.0),
                    ("negative.manufacturer", None),
                    ("negative.type", None),
                    ("positive.elevation", 0.0),
                    ("positive.id", None),
                    ("positive.latitude", 0.0),
                    ("positive.longitude", 0.0),
                    ("positive.manufacturer", None),
                    ("positive.type", None),
                    ("sample_rate", 4.0),
                    ("time_period.end", "2012-08-24T16:25:26+00:00"),
                    ("time_period.start", "2012-08-21T22:02:27+00:00"),
                    ("type", "electric"),
                    ("units", None),
                ]
            ),
        )

    def test_ey(self):
        self.assertDictEqual(
            self.header.ey_metadata.to_dict(single=True),
            OrderedDict(
                [
                    ("channel_number", 34),
                    ("component", "ey"),
                    ("data_quality.rating.value", 0),
                    ("dipole_length", 102.0),
                    ("filter.applied", [False]),
                    ("filter.name", []),
                    ("measurement_azimuth", 99.0),
                    ("measurement_tilt", 0.0),
                    ("negative.elevation", 0.0),
                    ("negative.id", None),
                    ("negative.latitude", 0.0),
                    ("negative.longitude", 0.0),
                    ("negative.manufacturer", None),
                    ("negative.type", None),
                    ("positive.elevation", 0.0),
                    ("positive.id", None),
                    ("positive.latitude", 0.0),
                    ("positive.longitude", 0.0),
                    ("positive.manufacturer", None),
                    ("positive.type", None),
                    ("sample_rate", 4.0),
                    ("time_period.end", "2012-08-24T16:25:26+00:00"),
                    ("time_period.start", "2012-08-21T22:02:27+00:00"),
                    ("type", "electric"),
                    ("units", None),
                ]
            ),
        )

    def test_hx(self):
        self.assertDictEqual(
            self.header.hx_metadata.to_dict(single=True),
            OrderedDict(
                [
                    ("channel_number", 31),
                    ("component", "hx"),
                    ("data_quality.rating.value", 0),
                    ("filter.applied", [False]),
                    ("filter.name", []),
                    ("location.elevation", 0.0),
                    ("location.latitude", 0.0),
                    ("location.longitude", 0.0),
                    ("measurement_azimuth", 9.0),
                    ("measurement_tilt", 0.0),
                    ("sample_rate", 4.0),
                    ("sensor.id", "2311-11"),
                    ("sensor.manufacturer", None),
                    ("sensor.type", None),
                    ("time_period.end", "2012-08-24T16:25:26+00:00"),
                    ("time_period.start", "2012-08-21T22:02:27+00:00"),
                    ("type", "magnetic"),
                    ("units", None),
                ]
            ),
        )

    def test_hy(self):
        self.assertDictEqual(
            self.header.hy_metadata.to_dict(single=True),
            OrderedDict(
                [
                    ("channel_number", 33),
                    ("component", "hy"),
                    ("data_quality.rating.value", 0),
                    ("filter.applied", [False]),
                    ("filter.name", []),
                    ("location.elevation", 0.0),
                    ("location.latitude", 0.0),
                    ("location.longitude", 0.0),
                    ("measurement_azimuth", 99.0),
                    ("measurement_tilt", 0.0),
                    ("sample_rate", 4.0),
                    ("sensor.id", "2311-11"),
                    ("sensor.manufacturer", None),
                    ("sensor.type", None),
                    ("time_period.end", "2012-08-24T16:25:26+00:00"),
                    ("time_period.start", "2012-08-21T22:02:27+00:00"),
                    ("type", "magnetic"),
                    ("units", None),
                ]
            ),
        )

    def test_hz(self):
        self.assertDictEqual(
            self.header.hz_metadata.to_dict(single=True),
            OrderedDict(
                [
                    ("channel_number", 35),
                    ("component", "hz"),
                    ("data_quality.rating.value", 0),
                    ("filter.applied", [False]),
                    ("filter.name", []),
                    ("location.elevation", 0.0),
                    ("location.latitude", 0.0),
                    ("location.longitude", 0.0),
                    ("measurement_azimuth", 0.0),
                    ("measurement_tilt", 0.0),
                    ("sample_rate", 4.0),
                    ("sensor.id", "2311-11"),
                    ("sensor.manufacturer", None),
                    ("sensor.type", None),
                    ("time_period.end", "2012-08-24T16:25:26+00:00"),
                    ("time_period.start", "2012-08-21T22:02:27+00:00"),
                    ("type", "magnetic"),
                    ("units", None),
                ]
            ),
        )

    def test_latitude(self):
        self.assertEqual(self.header.latitude, 39.282)

    def test_longitude(self):
        self.assertEqual(self.header.longitude, -108.1582)

    def test_n_channels(self):
        self.assertEqual(self.header.n_channels, 5)

    def test_n_samples(self):
        self.assertEqual(self.header.n_samples, 955916)

    def test_run_id(self):
        self.assertEqual(self.header.run_id, "rgr003a")

    def test_run_metadata(self):
        self.assertDictEqual(
            self.header.run_metadata.to_dict(single=True),
            OrderedDict(
                [
                    ("channels_recorded_auxiliary", []),
                    ("channels_recorded_electric", ["ex", "ey"]),
                    ("channels_recorded_magnetic", ["hx", "hy", "hz"]),
                    ("data_logger.firmware.author", None),
                    ("data_logger.firmware.name", None),
                    ("data_logger.firmware.version", None),
                    ("data_logger.id", "2311-11"),
                    ("data_logger.manufacturer", None),
                    ("data_logger.timing_system.drift", 0.0),
                    ("data_logger.timing_system.type", "GPS"),
                    ("data_logger.timing_system.uncertainty", 0.0),
                    ("data_logger.type", None),
                    ("data_type", "BBMT"),
                    ("id", "rgr003a"),
                    ("sample_rate", 4.0),
                    ("time_period.end", "2012-08-24T16:25:26+00:00"),
                    ("time_period.start", "2012-08-21T22:02:27+00:00"),
                ]
            ),
        )

    def test_sample_rate(self):
        self.assertEqual(self.header.sample_rate, 4.0)

    def test_site_id(self):
        self.assertEqual(self.header.site_id, "003")

    def test_start(self):
        self.assertEqual(self.header.start, "2012-08-21T22:02:27+00:00")

    def test_station_metadata(self):
        self.assertDictEqual(
            self.header.station_metadata.to_dict(single=True),
            OrderedDict(
                [
                    ("acquired_by.name", None),
                    ("channels_recorded", ["ex", "ey", "hx", "hy", "hz"]),
                    ("data_type", "BBMT"),
                    ("geographic_name", None),
                    ("id", "003"),
                    ("location.declination.model", "WMM"),
                    ("location.declination.value", 0.0),
                    ("location.elevation", 1803.07),
                    ("location.latitude", 39.282),
                    ("location.longitude", -108.1582),
                    ("orientation.method", None),
                    ("orientation.reference_frame", "geographic"),
                    ("provenance.creation_time", "1980-01-01T00:00:00+00:00"),
                    ("provenance.software.author", None),
                    ("provenance.software.name", None),
                    ("provenance.software.version", None),
                    ("provenance.submitter.email", None),
                    ("provenance.submitter.organization", None),
                    ("release_license", "CC0-1.0"),
                    ("run_list", ["rgr003a"]),
                    ("time_period.end", "2012-08-24T16:25:26+00:00"),
                    ("time_period.start", "2012-08-21T22:02:27+00:00"),
                ]
            ),
        )

    def test_survey_id(self):
        self.assertEqual(self.header.survey_id, "RGR")

    def test_survey_metadata(self):
        od = OrderedDict(
            [
                ("citation_dataset.doi", None),
                ("citation_journal.doi", None),
                ("datum", "WGS84"),
                ("geographic_name", None),
                ("id", "RGR"),
                ("name", None),
                ("northwest_corner.latitude", 0.0),
                ("northwest_corner.longitude", 0.0),
                ("project", None),
                ("project_lead.email", None),
                ("project_lead.organization", None),
                ("release_license", "CC0-1.0"),
                ("southeast_corner.latitude", 0.0),
                ("southeast_corner.longitude", 0.0),
                ("summary", None),
                ("time_period.end_date", "1980-01-01"),
                ("time_period.start_date", "1980-01-01"),
            ]
        )
        for key in self.header.survey_metadata.to_dict(single=True).keys():
            if "mth5" in key:
                continue
            with self.subTest(key):
                self.assertEqual(
                    self.header.survey_metadata.get_attr_from_name(key), od[key]
                )

    def test_write_header(self):
        original = [
            "SurveyID: RGR",
            "SiteID: 003",
            "RunID: rgr003a",
            "SiteLatitude: 39.282",
            "SiteLongitude: -108.1582",
            "SiteElevation: 1803.0",
            "AcqStartTime: 2012-08-21T22:02:27+00:00",
            "AcqStopTime: 2012-08-24T16:25:26+00:00",
            "AcqSmpFreq: 4.0",
            "AcqNumSmp: 955916",
            "Nchan: 5",
            "Channel coordinates relative to geographic north",
            "ChnSettings:",
            "ChnNum ChnID InstrumentID Azimuth Dipole_Length",
            "31      hx    2311-11         9.0           0.0",
            "32      ex    2311-11         9.0         100.0",
            "33      hy    2311-11        99.0           0.0",
            "34      ey    2311-11        99.0         102.0",
            "35      hz    2311-11         0.0           0.0",
            "MissingDataFlag: 1.000e+09",
            "DataSet:",
        ]

        lines = self.header.write_metadata()

        self.assertListEqual(original, lines)


# =============================================================================
# Run
# =============================================================================
if __name__ == "__main__":
    unittest.main()



# package file
from .header import AsciiMetadata
from .usgs_ascii import USGSascii, read_ascii
from .usgs_ascii_collection import USGSasciiCollection

__all__ = ["AsciiMetadata", "USGSascii", "read_ascii", "USGSasciiCollection"]



from .header import Header
from .base import TSReaderBase
from .native import NativeReader
from .segmented import DecimatedSegmentedReader
from .contiguous import DecimatedContinuousReader
from .phx_json import ConfigJSON, ReceiverMetadataJSON

__all__ = [
    "Header",
    "TSReaderBase",
    "NativeReader",
    "DecimatedSegmentedReader",
    "DecimatedContinuousReader",
    "ConfigJSON",
    "ReceiverMetadataJSON",
]



# -*- coding: utf-8 -*-
"""
Created on Fri Mar 11 15:24:28 2022

@author: jpeacock
"""

# =============================================================================
# Imports
# =============================================================================
import unittest
from pathlib import Path
from collections import OrderedDict

from mth5.mth5 import MTH5

from mt_metadata.transfer_functions.core import TF
from mt_metadata import TF_XML


fn_path = Path(__file__).parent
# =============================================================================


class TestTFGroup(unittest.TestCase):
    @classmethod
    def setUpClass(self):

        self.maxDiff = None
        self.fn = fn_path.joinpath("test.mth5")
        self.mth5_obj = MTH5(file_version="0.2.0")
        self.mth5_obj.open_mth5(self.fn, mode="a")

        self.tf_obj = TF(TF_XML)
        self.tf_obj.read_tf_file()

        self.tf_group = self.mth5_obj.add_transfer_function(self.tf_obj)
        self.tf_h5 = self.mth5_obj.get_transfer_function(
            self.tf_obj.station,
            self.tf_obj.tf_id,
            survey=self.tf_obj.survey_metadata.id,
        )

    def test_survey_metadata(self):
        meta_dict = OrderedDict(
            [
                ("acquired_by.author", "National Geoelectromagnetic Facility"),
                (
                    "citation_dataset.doi",
                    "doi:10.17611/DP/EMTF/USMTARRAY/SOUTH",
                ),
                (
                    "citation_dataset.title",
                    "USMTArray South Magnetotelluric Transfer Functions",
                ),
                ("citation_dataset.year", "2020-2023"),
                ("citation_journal.doi", None),
                (
                    "comments",
                    "The USMTArray-CONUS South campaign was carried out through a cooperative agreement between\nthe U.S. Geological Survey (USGS) and Oregon State University (OSU). A subset of 40 stations\nin the SW US were funded through NASA grant 80NSSC19K0232.\nLand permitting, data acquisition, quality control and field processing were\ncarried out by Green Geophysics with project management and instrument/engineering\nsupport from OSU and Chaytus Engineering, respectively.\nProgram oversight, definitive data processing and data archiving were provided\nby the USGS Geomagnetism Program and the Geology, Geophysics and Geochemistry Science Centers.\nWe thank the U.S. Forest Service, the Bureau of Land Management, the National Park Service,\nthe Department of Defense, numerous state land offices and the many private landowners\nwho permitted land access to acquire the USMTArray data.",
                ),
                ("country", ["USA"]),
                ("datum", "WGS84"),
                ("geographic_name", "CONUS South"),
                ("id", "CONUS South"),
                ("name", None),
                ("northwest_corner.latitude", 0.0),
                ("northwest_corner.longitude", 0.0),
                ("project", "USMTArray"),
                ("project_lead.email", None),
                ("project_lead.organization", None),
                ("release_license", "CC0-1.0"),
                ("southeast_corner.latitude", 0.0),
                ("southeast_corner.longitude", 0.0),
                ("summary", "Magnetotelluric Transfer Functions"),
                ("time_period.end_date", "2020-10-07"),
                ("time_period.start_date", "2020-09-20"),
            ]
        )

        h5_meta_dict = self.tf_h5.survey_metadata.to_dict(single=True)
        h5_meta_dict.pop("mth5_type")
        h5_meta_dict.pop("hdf5_reference")

        self.assertDictEqual(meta_dict, h5_meta_dict)

    def test_station_metadta(self):

        meta_dict = OrderedDict(
            [
                ("acquired_by.author", "National Geoelectromagnetic Facility"),
                ("channels_recorded", ["ex", "ey", "hx", "hy", "hz"]),
                ("data_type", "mt"),
                ("fdsn.id", "USMTArray.NMX20.2020"),
                ("geographic_name", "Nations Draw, NM, USA"),
                ("id", "NMX20"),
                ("location.datum", "WGS84"),
                ("location.declination.epoch", "2020.0"),
                ("location.declination.model", "WMM"),
                ("location.declination.value", 9.0899999999999999),
                ("location.elevation", 1940.05),
                ("location.latitude", 34.470528),
                ("location.longitude", -108.712288),
                ("orientation.angle_to_geographic_north", 0.0),
                ("orientation.method", None),
                ("orientation.reference_frame", "geographic"),
                ("provenance.creation_time", "2021-03-17T14:47:44+00:00"),
                ("provenance.software.author", None),
                (
                    "provenance.software.name",
                    "EMTF File Conversion Utilities 4.0",
                ),
                ("provenance.software.version", None),
                ("provenance.submitter.author", "Anna Kelbert"),
                ("provenance.submitter.email", "akelbert@usgs.gov"),
                (
                    "provenance.submitter.organization",
                    "U.S. Geological Survey, Geomagnetism Program",
                ),
                ("release_license", "CC0-1.0"),
                ("run_list", ["NMX20a", "NMX20b"]),
                ("time_period.end", "2020-10-07T20:28:00+00:00"),
                ("time_period.start", "2020-09-20T19:03:06+00:00"),
                ("transfer_function.coordinate_system", "geopgraphic"),
                ("transfer_function.id", "NMX20"),
                ("transfer_function.processed_date", None),
                ("transfer_function.processing_parameters", ["{type: None}"]),
                (
                    "transfer_function.remote_references",
                    [
                        "NMX20b",
                        "NMX20",
                        "NMW20",
                        "COR21",
                        "NMY21-NMX20b",
                        "NMX20",
                        "UTS18",
                    ],
                ),
                ("transfer_function.runs_processed", ["NMX20a", "NMX20b"]),
                ("transfer_function.sign_convention", "exp(+ i\\omega t)"),
                ("transfer_function.units", None),
            ]
        )

        self.assertDictEqual(
            meta_dict, self.tf_h5.station_metadata.to_dict(single=True)
        )

    def test_runs(self):

        for run1, run2 in zip(
            self.tf_h5.station_metadata.runs, self.tf_obj.station_metadata.runs
        ):
            with self.subTest(run1.id):
                run1.data_logger.firmware.author = None
                rd1 = run1.to_dict(single=True)
                rd1.pop("mth5_type")
                rd1.pop("hdf5_reference")

                rd2 = run2.to_dict(single=True)
                rd2.pop("mth5_type")
                rd2.pop("hdf5_reference")
                self.assertDictEqual(rd1, rd2)

    def test_channels(self):

        for run1, run2 in zip(
            self.tf_h5.station_metadata.runs, self.tf_obj.station_metadata.runs
        ):
            for ch1 in run1.channels:
                ch2 = run2.get_channel(ch1.component)
                with self.subTest(f"{run1.id}_{ch1.component}"):

                    chd1 = ch1.to_dict(single=True)
                    chd1.pop("mth5_type")
                    chd1.pop("hdf5_reference")

                    chd2 = ch2.to_dict(single=True)
                    chd2.pop("mth5_type")
                    chd2.pop("hdf5_reference")
                    self.assertDictEqual(chd1, chd2)

    def test_estimates(self):

        for estimate in [
            "transfer_function",
            "transfer_function_error",
            "inverse_signal_power",
            "residual_covariance",
        ]:

            with self.subTest(estimate):
                est1 = getattr(self.tf_obj, estimate)
                est2 = getattr(self.tf_h5, estimate)

                self.assertTrue((est1.to_numpy() == est2.to_numpy()).all())

    def test_period(self):
        self.assertTrue((self.tf_obj.period == self.tf_h5.period).all())

    def test_tf_summary(self):
        self.mth5_obj.tf_summary.clear_table()
        self.mth5_obj.tf_summary.summarize()

        with self.subTest("test shape"):
            self.assertEqual(self.mth5_obj.tf_summary.shape, (1,))
        true_dict = dict(
            [
                ("station", b"NMX20"),
                ("survey", b"CONUS_South"),
                ("latitude", 34.470528),
                ("longitude", -108.712288),
                ("elevation", 1940.05),
                ("tf_id", b"NMX20"),
                ("units", b"none"),
                ("has_impedance", True),
                ("has_tipper", True),
                ("has_covariance", True),
                ("period_min", 4.6545500000000004),
                ("period_max", 29127.110000000001),
            ]
        )
        for name in self.mth5_obj.tf_summary.dtype.names:
            if "reference" in name:
                continue
            with self.subTest(f"test {name}"):
                self.assertEqual(
                    self.mth5_obj.tf_summary.array[name][0], true_dict[name]
                )

    @classmethod
    def tearDownClass(self):
        self.mth5_obj.close_mth5()
        self.fn.unlink()



# -*- coding: utf-8 -*-
"""
Created on Sat Sep 10 18:19:12 2022

@author: jpeacock
"""

# =============================================================================
# Imports
# =============================================================================
import unittest
from collections import OrderedDict
from pathlib import Path

import pandas as pd
import numpy as np

from mth5.io.zen import Z3DCollection

# =============================================================================


@unittest.skipIf(
    "peacock" not in str(Path(__file__).as_posix()), "local files"
)
class TestZ3DCollection(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.zc = Z3DCollection(
            r"c:\Users\jpeacock\OneDrive - DOI\mt\example_z3d_data"
        )

        self.df = self.zc.to_dataframe([256, 4096])
        self.runs = self.zc.get_runs([256, 4096])

        self.station = self.df.station.unique()[0]
        self.maxDiff = None

    def test_file_path(self):
        self.assertIsInstance(self.zc.file_path, Path)

    def test_get_files(self):
        self.assertEqual(10, len(self.zc.get_files(self.zc.file_ext)))

    def test_df_columns(self):
        self.assertListEqual(
            self.zc._columns,
            self.df.columns.to_list(),
        )

    def test_df_shape(self):
        self.assertEqual(self.df.shape, (10, 14))

    def test_df_types(self):
        self.df = self.zc._set_df_dtypes(self.df)
        with self.subTest("start"):
            self.assertTrue(
                self.df.start.dtype.type
                == pd._libs.tslibs.timestamps.Timestamp
            )
        with self.subTest("end"):
            self.assertTrue(
                self.df.end.dtype.type == pd._libs.tslibs.timestamps.Timestamp
            )

        with self.subTest("instrument_id"):
            self.assertTrue(self.df.instrument_id.dtype.type == np.object_)

        with self.subTest("calibration_fn"):
            self.assertTrue(self.df.calibration_fn.dtype.type == np.object_)

    def test_survey_id(self):
        self.assertTrue((self.df.survey == "").all())

    def test_df_run_names_256(self):
        self.assertEqual(
            "sr256_0002", self.df[self.df.sample_rate == 256].run.unique()[0]
        )

    def test_df_run_names_4096(self):
        self.assertEqual(
            "sr4096_0001", self.df[self.df.sample_rate == 4096].run.unique()[0]
        )

    def test_run_dtype(self):
        self.assertIsInstance(self.runs, OrderedDict)

    def test_run_elements(self):
        for key, rdf in self.runs[self.station].items():
            with self.subTest(key):
                test_rdf = self.df[self.df.run == key]
                self.assertTrue(
                    (
                        self.df[self.df.run == key]
                        .iloc[0:8]
                        .eq(rdf)
                        .all(axis=0)
                        .all()
                    )
                )


# =============================================================================
# Run
# =============================================================================
if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
LEMI 424 Collection
====================

Collection of TXT files combined into runs

Created on Wed Aug 31 10:32:44 2022

@author: jpeacock
"""

# =============================================================================
# Imports
# =============================================================================
import pandas as pd

from mth5.io.collection import Collection
from mth5.io.usgs_ascii import USGSascii

# =============================================================================


class USGSasciiCollection(Collection):
    """
    Collection of USGS ASCII files.

    .. code-block:: python

        >>> from mth5.io.usgs_ascii import USGSasciiCollection
        >>> lc = USGSasciiCollection(r"/path/to/ascii/files")
        >>> run_dict = lc.get_runs(1)


    """

    def __init__(self, file_path=None, **kwargs):
        super().__init__(file_path=file_path, **kwargs)
        self.file_ext = "asc"

    def to_dataframe(
        self, sample_rates=[4], run_name_zeros=4, calibration_path=None
    ):
        """
        Create a data frame of each TXT file in a given directory.

        .. note:: If a run name is already present it will not be overwritten

        :param sample_rates: sample rate to get, defaults to [4]
        :type sample_rates: int or list, optional
        :param run_name_zeros: number of zeros to assing to the run name,
         defaults to 4
        :type run_name_zeros: int, optional
        :param calibration_path: path to calibration files, defaults to None
        :type calibration_path: string or Path, optional
        :return: Dataframe with information of each TXT file in the given
         directory.
        :rtype: :class:`pandas.DataFrame`

        :Example:

            >>> from mth5.io.usgs_ascii import USGSasciiCollection
            >>> lc = USGSasciiCollection("/path/to/ascii/files")
            >>> ascii_df = lc.to_dataframe()

        """

        entries = []
        for fn in self.get_files(self.file_ext):
            asc_obj = USGSascii(fn)
            asc_obj.read_metadata()

            entry = {}
            entry["survey"] = asc_obj.survey_metadata.id
            entry["station"] = asc_obj.station_metadata.id
            entry["run"] = asc_obj.run_metadata.id
            entry["start"] = asc_obj.start
            entry["end"] = asc_obj.end
            entry["channel_id"] = 1
            entry["component"] = ",".join(
                asc_obj.run_metadata.channels_recorded_all
            )
            entry["fn"] = fn
            entry["sample_rate"] = asc_obj.sample_rate
            entry["file_size"] = asc_obj.file_size
            entry["n_samples"] = int(asc_obj.n_samples)
            entry["sequence_number"] = 0
            entry["instrument_id"] = asc_obj.run_metadata.data_logger.id
            entry["calibration_fn"] = None

            entries.append(entry)

        # make pandas dataframe and set data types
        df = self._sort_df(
            self._set_df_dtypes(pd.DataFrame(entries)), run_name_zeros
        )

        return df

    def assign_run_names(self, df, zeros=4):
        """
        Assign run names based on start and end times, checks if a file has
        the same start time as the last end time.

        Run names are assigned as sr{sample_rate}_{run_number:0{zeros}}. Only
        if the run name is not assigned already.

        :param df: Dataframe with the appropriate columns
        :type df: :class:`pandas.DataFrame`
        :param zeros: number of zeros in run name, defaults to 4
        :type zeros: int, optional
        :return: Dataframe with run names
        :rtype: :class:`pandas.DataFrame`

        """

        for station in df.station.unique():
            count = 1
            for row in (
                df[df.station == station].sort_values("start").itertuples()
            ):
                if row.run is None:
                    df.loc[
                        row.Index, "run"
                    ] = f"sr{row.sample_rate}_{count:0{zeros}}"
                df.loc[row.Index, "sequence_number"] = count
                count += 1

        return df



# -*- coding: utf-8 -*-
"""
Module to read and parse native Phoenix Geophysics data formats of the 
MTU-5C Family.

This module implements Streamed readers for decimated continuos time series
formats of the MTU-5C family.

:author: Jorge Torres-Solis

Revised 2022 by J. Peacock 
"""

# =============================================================================
# Imports
# =============================================================================

import numpy as np

from mth5.io.phoenix.readers import TSReaderBase
from mth5.timeseries import ChannelTS

# =============================================================================


class DecimatedContinuousReader(TSReaderBase):
    """
    Class to create a streamer for continuous decimated time series,
    i.e. 'td_150', 'td_30'

    These files have no sub header information.

    """

    def __init__(self, path, num_files=1, report_hw_sat=False, **kwargs):
        # Init the base class
        super().__init__(
            path,
            num_files=num_files,
            header_length=128,
            report_hw_sat=report_hw_sat,
            **kwargs,
        )

        self.unpack_header(self.stream)
        self.subheader = {}

    @property
    def segment_start_time(self):
        """
        estimate the segment start time based on sequence number

        The first sequence starts 1 second later than the set start time due
        to initiation within the data logger

        :return: start time of the recording
        :rtype: :class:`mt_metadata.utils.mttime.MTime`

        """

        if self.file_sequence == 1:
            return self.recording_start_time + 1
        else:
            return self.recording_start_time + (
                self.frag_period * (self.file_sequence - 1)
            )

    @property
    def segment_end_time(self):
        """
        estimate end time

        The first sequence starts 1 second later than the set start time due
        to initiation within the data logger

        :return: estimated end time from number of samples
        :rtype: :class:`mt_metadata.utils.mttime.MTime`

        """

        return self.segment_start_time + (self.max_samples / self.sample_rate)

    # need a read and read sequence
    def read(self):
        """
        Read in the full data from the file given.

        :return: single channel data array
        :rtype: :class:`numpy.ndarray`

        """

        self.stream.seek(self.header_length)
        return np.fromfile(self.stream, dtype=np.float32)

    def read_sequence(self, start=0, end=None):
        """
        Read a sequence of files

        :param start: starting index in the sequence, defaults to 0
        :type start: integer, optional
        :param end: eneding index in the sequence to read, defaults to None
        :type end: integer, optional
        :return: data within the given sequence range
        :rtype: :class:`numpy.ndarray`

        """
        data = np.array([], dtype=np.float32)
        for ii, fn in enumerate(self.sequence_list[slice(start, end)], start):
            self._open_file(fn)
            self.unpack_header(self.stream)
            ts = self.read()
            data = np.append(data, ts)
            self.seq = ii

        self.logger.debug("Read %s sequences", self.seq + 1)
        return data

    def to_channel_ts(self):
        """
        convert to a ChannelTS object

        :return: DESCRIPTION
        :rtype: TYPE

        """
        data = self.read_sequence()
        ch_metadata = self.channel_metadata()
        return ChannelTS(
            channel_type=ch_metadata.type,
            data=data,
            channel_metadata=ch_metadata,
            run_metadata=self.run_metadata(),
            station_metadata=self.station_metadata(),
        )



# -*- coding: utf-8 -*-
"""
Created on Wed Dec  1 12:18:08 2021

@author: jpeacock
"""
# =============================================================================
# Imports
# =============================================================================
import unittest
import numpy as np

from mt_metadata.timeseries.filters import PoleZeroFilter
from mth5.timeseries import ChannelTS

# =============================================================================
#
# =============================================================================


class TestRemoveResponse(unittest.TestCase):
    """
    Test remove response, make a fake signal add some trends, 
    """

    @classmethod
    def setUpClass(self):
        # pole zero filter
        pz = PoleZeroFilter(
            units_in="volts", units_out="nanotesla", name="instrument_response"
        )
        pz.poles = [
            (-6.283185 + 10.882477j),
            (-6.283185 - 10.882477j),
            (-12.566371 + 0j),
        ]
        pz.zeros = []
        pz.normalization_factor = 18244400

        # channel properties
        self.channel = ChannelTS()
        self.channel.channel_metadata.filter.applied = [False]
        self.channel.channel_metadata.filter.name = ["instrument_response"]
        self.channel.channel_metadata.component = "hx"
        self.channel.channel_response_filter.filters_list.append(pz)
        self.channel.sample_rate = 1
        n_samples = 4096
        self.t = np.arange(n_samples) * self.channel.sample_interval
        # make a random signal
        self.example_ts = np.sum(
            [
                np.cos(2 * np.pi * w * self.t + phi)
                for w, phi in zip(
                    np.logspace(-4, self.channel.sample_rate / 2, 20),
                    np.random.rand(20),
                )
            ],
            axis=0,
        )

        # multiply by filter response
        f = np.fft.rfftfreq(self.t.size, self.channel.sample_interval)
        response_ts = np.fft.irfft(
            np.fft.rfft(self.example_ts) * pz.complex_response(f)[::-1]
        )

        # add in a linear trend
        self.channel.ts = (0.3 * self.t) + response_ts

        self.calibrated_ts = self.channel.remove_instrument_response()

    def test_return_type(self):
        self.assertIsInstance(self.calibrated_ts, ChannelTS)

    def test_applied(self):
        self.assertTrue(
            (np.array(self.calibrated_ts.channel_metadata.filter.applied) == True).all()
        )

    def test_returned_metadata(self):
        with self.subTest("component"):
            self.assertEqual(self.channel.component, self.calibrated_ts.component)
        with self.subTest("sample rate"):
            self.assertEqual(self.channel.sample_rate, self.calibrated_ts.sample_rate)

    def test_calibration(self):

        original_ts = self.example_ts - self.example_ts.mean()
        std = (self.calibrated_ts.ts - original_ts).std() / original_ts.max()
        self.assertLessEqual(std, 0.075)


# =============================================================================
# run
# =============================================================================
if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Created on Wed Dec 23 16:53:55 2020

:author: Jared Peacock

:license: MIT

"""
# =============================================================================
# Imports
# =============================================================================
import weakref
import copy

import h5py
import numpy as np
import pandas as pd

from mth5.utils.exceptions import MTH5TableError
from mth5.utils.mth5_logger import setup_logger

# =============================================================================
# MTH5 Table Class
# =============================================================================


class MTH5Table:
    """
    Use the underlying NumPy basics, there are simple actions in this table,
    if a user wants to use something more sophisticated for querying they
    should try using a pandas table.  In this case entries in the table
    are more difficult to change and datatypes need to be kept track of.

    """

    def __init__(self, hdf5_dataset):
        self.logger = setup_logger(f"{__name__}.{self.__class__.__name__}")

        self.hdf5_reference = None
        if isinstance(hdf5_dataset, h5py.Dataset):
            self.array = weakref.ref(hdf5_dataset)()
            self.hdf5_reference = hdf5_dataset.ref
        else:
            msg = "Input must be a h5py.Dataset not {0}".format(type(hdf5_dataset))
            self.logger.error(msg)
            raise MTH5TableError(msg)

    def __str__(self):
        """
        return a string that shows the table in text form

        :return: text representation of the table
        :rtype: string

        """
        # if the array is empty
        if self.array.size == 0:
            length_dict = dict([(key, len(str(key))) for key in list(self.dtype.names)])
            lines = [
                " | ".join(
                    ["index"]
                    + [
                        "{0:^{1}}".format(name, length_dict[name])
                        for name in list(self.dtype.names)
                    ]
                )
            ]
            lines.append("-" * len(lines[0]))
            return "\n".join(lines)

        length_dict = dict(
            [
                (key, max([len(str(b)) for b in self.array[key]]))
                for key in list(self.dtype.names)
            ]
        )
        lines = [
            " | ".join(
                ["index"]
                + [
                    "{0:^{1}}".format(name, length_dict[name])
                    for name in list(self.dtype.names)
                ]
            )
        ]
        lines.append("-" * len(lines[0]))

        for ii, row in enumerate(self.array):
            line = ["{0:^5}".format(ii)]
            for element, key in zip(row, list(self.dtype.names)):
                if isinstance(element, (np.bytes_)):
                    element = element.decode()
                try:
                    line.append("{0:^{1}}".format(element, length_dict[key]))

                except TypeError as error:
                    if isinstance(element, h5py.h5r.Reference):
                        msg = "{0}: Cannot represent h5 reference as a string"
                        self.logger.debug(msg.format(error))
                        line.append(
                            "{0:^{1}}".format(
                                "<HDF5 object reference>", length_dict[key]
                            )
                        )
                    else:
                        self.logger.exception(f"{error}")

            lines.append(" | ".join(line))
        return "\n".join(lines)

    def __repr__(self):
        return self.__str__()

    def __eq__(self, other):
        if isinstance(other, MTH5Table):
            return self.array == other.array
        elif isinstance(other, h5py.Dataset):
            return self.array == other
        else:
            msg = "Cannot compare type={0}".format(type(other))
            self.logger.error(msg)
            raise TypeError(msg)

    def __ne__(self, other):
        return not self.__eq__(other)

    def __len__(self):
        return self.array.shape[0]

    @property
    def dtype(self):
        try:
            return self.array.dtype
        except AttributeError as error:
            msg = "{0}, dataframe is not initiated yet".format(error)
            self.logger.warning(msg)
            return None

    def check_dtypes(self, other_dtype):
        """
        Check to make sure datatypes match
        """

        if self.dtype == other_dtype:
            return True

        return False

    @property
    def shape(self):
        return self.array.shape

    @property
    def nrows(self):
        return self.array.shape[0]

    def locate(self, column, value, test="eq"):
        """

        locate index where column is equal to value
        :param column: DESCRIPTION
        :type column: TYPE
        :param value: DESCRIPTION
        :type value: TYPE
        :type test: type of test to try
        * 'eq': equals
        * 'lt': less than
        * 'le': less than or equal to
        * 'gt': greater than
        * 'ge': greater than or equal to.
        * 'be': between or equal to
        * 'bt': between

        If be or bt input value as a list of 2 values

        :return: DESCRIPTION
        :rtype: TYPE

        """
        if isinstance(value, str):
            value = np.bytes_(value)

        # use numpy datetime for testing against time.
        if column in ["start", "end", "start_date", "end_date"]:
            test_array = self.array[column].astype(np.datetime64)
            value = np.datetime64(value)
        else:
            test_array = self.array[column]

        if test == "eq":
            index_values = np.where(test_array == value)[0]
        elif test == "lt":
            index_values = np.where(test_array < value)[0]
        elif test == "le":
            index_values = np.where(test_array <= value)[0]
        elif test == "gt":
            index_values = np.where(test_array > value)[0]
        elif test == "ge":
            index_values = np.where(test_array >= value)[0]
        elif test == "be":
            if not isinstance(value, (list, tuple, np.ndarray)):
                msg = "If testing for between value must be an iterable of length 2."
                self.logger.error(msg)
                raise ValueError(msg)

            index_values = np.where((test_array > value[0]) & (test_array < value[1]))[
                0
            ]
        else:
            raise ValueError("Test {0} not understood".format(test))

        return index_values

    def add_row(self, row, index=None):
        """
        Add a row to the table.

        row must be of the same data type as the table


        :param row: row entry for the table
        :type row: TYPE

        :param index: index of row to add
        :type index: integer, if None is given then the row is added to the
                     end of the array

        :return: index of the row added
        :rtype: integer

        """

        if not isinstance(row, (np.ndarray)):
            msg = "Input must be an numpy.ndarray" + "not {0}".format(type(row))
        if isinstance(row, np.ndarray):
            if not self.check_dtypes(row.dtype):
                msg = "{0}\nInput dtypes:\n{1}\n\nTable dtypes:\n{2}".format(
                    "Data types are not equal:", row.dtype, self.dtype
                )
                self.logger.error(msg)
                raise ValueError(msg)

        if index is None:
            index = self.nrows
            if self.nrows == 1:
                match = True
                null_array = np.empty(1, dtype=self.dtype)
                for name in self.dtype.names:
                    if "reference" in name:
                        continue
                    if self.array[name][0] != null_array[name][0]:
                        match = False
                        break

                if match:
                    index = 0
                else:
                    new_shape = tuple([self.nrows + 1] + [ii for ii in self.shape[1:]])
                    self.array.resize(new_shape)
            else:
                new_shape = tuple([self.nrows + 1] + [ii for ii in self.shape[1:]])
                self.array.resize(new_shape)
        # add the row
        self.array[index] = row
        self.logger.debug("Added row as index {0} with values {1}".format(index, row))

        return index

    def update_row(self, entry):
        """
        Update an entry by first locating the index and then rewriting the entry.

        :param entry: numpy array with same datatype as the table
        :type entry: np.ndarray

        :return: row index.

        This doesn't work because you cannot test for hdf5_reference, should use
        add row and locate by index.

        """
        try:
            row_index = self.locate("hdf5_reference", entry["hdf5_reference"])[0]
            return self.add_row(entry, index=row_index)

        except IndexError:
            self.logger.debug("Could not find row, adding a new one")
            return self.add_row(entry)

    def remove_row(self, index):
        """
        Remove a row

        .. note:: that there is not index value within the array, so the
                  indexing is on the fly.  A user should use the HDF5
                  reference instead of index number that is the safest and
                  most robust method.

        :param index: DESCRIPTION
        :type index: TYPE
        :return: DESCRIPTION
        :rtype: TYPE

        This isn't as easy as just deleteing an element.
        Need to delete the element from the weakly referenced array and then
        set the summary table dataset to the new array.

        So set to a null array for now until a more clever option is found.

        """
        null_array = np.empty((1,), dtype=self.dtype)
        try:
            return self.add_row(null_array, index=index)

        except IndexError as error:
            msg = "Could not find index {0} in shape {1}".format(index, self.shape())
            self.logger.exception(msg)
            raise IndexError(f"{error}\n{msg}")

    def to_dataframe(self):
        """
        Convert the table into a :class:`pandas.DataFrame` object.

        :return: convert table into a :class:`pandas.DataFrame` with the
                 appropriate data types.
        :rtype: :class:`pandas.DataFrame`

        """

        df = pd.DataFrame(self.array[()])
        for key in [
            "station",
            "run",
            "component",
            "measurement_type",
            "units",
        ]:
            setattr(df, key, getattr(df, key).str.decode("utf-8"))
        df.start = pd.to_datetime(df.start.str.decode("utf-8"))
        df.end = pd.to_datetime(df.end.str.decode("utf-8"))

        return df

    def clear_table(self):
        """
        clear a table, 
        
        Basically delete the table and start over
        :return: DESCRIPTION
        :rtype: TYPE

        """

        dtype = copy.deepcopy(self.dtype)

        root = self.array.parent
        name = self.array.name.split("/")[-1]
        ds_options = {
            "compression": self.array.compression,
            "compression_opts": self.array.compression_opts,
            "shuffle": self.array.shuffle,
            "fletcher32": self.array.fletcher32,
        }

        del root[name]

        self.array = root.create_dataset(
            name, (1,), maxshape=(None,), dtype=dtype, **ds_options
        )



# -*- coding: utf-8 -*-
"""
Created on Sat Sep 10 21:00:28 2022

@author: jpeacock
"""

# =============================================================================
# Imports
# =============================================================================
import unittest
from pathlib import Path

import numpy as np

from mth5.io.zen import Z3DHeader

# =============================================================================


@unittest.skipIf(
    "peacock" not in str(Path(__file__).as_posix()), "local files"
)
class TestZ3DHeader(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.fn = r"c:\Users\jpeacock\OneDrive - DOI\mt\example_z3d_data\bm100_20220517_131017_256_EY.Z3D"
        self.z3d_obj = Z3DHeader(fn=self.fn)
        self.z3d_obj.read_header()

    def test_ad_gain(self):
        self.assertEqual(getattr(self.z3d_obj, "ad_gain"), 1.0)

    def test_ad_rate(self):
        self.assertEqual(getattr(self.z3d_obj, "ad_rate"), 256.0)

    def test_alt(self):
        self.assertEqual(getattr(self.z3d_obj, "alt"), 1456.3)

    def test_attenchannelsmask(self):
        self.assertEqual(getattr(self.z3d_obj, "attenchannelsmask"), "0x80")

    def test_box_number(self):
        self.assertEqual(getattr(self.z3d_obj, "box_number"), 24.0)

    def test_box_serial(self):
        self.assertEqual(
            getattr(self.z3d_obj, "box_serial"), "0x0000010013A20040"
        )

    def test_ch_factor(self):
        self.assertEqual(
            getattr(self.z3d_obj, "ch_factor"), 9.536743164062e-10
        )

    def test_channel(self):
        self.assertEqual(getattr(self.z3d_obj, "channel"), 5.0)

    def test_channelgain(self):
        self.assertEqual(getattr(self.z3d_obj, "channelgain"), 1.0)

    def test_channelserial(self):
        self.assertEqual(getattr(self.z3d_obj, "channelserial"), "0xD474777C")

    def test_data_logger(self):
        self.assertEqual(getattr(self.z3d_obj, "data_logger"), "ZEN024")

    def test_duty(self):
        self.assertEqual(getattr(self.z3d_obj, "duty"), 32767.0)

    def test_dutynormalized(self):
        self.assertEqual(getattr(self.z3d_obj, "dutynormalized"), np.inf)

    def test_dutyoff(self):
        self.assertEqual(getattr(self.z3d_obj, "dutyoff"), 1.0)

    def test_fn(self):
        self.assertEqual(getattr(self.z3d_obj, "fn"), self.fn)

    def test_fpga_buildnum(self):
        self.assertEqual(getattr(self.z3d_obj, "fpga_buildnum"), 1125.0)

    def test_gpsweek(self):
        self.assertEqual(getattr(self.z3d_obj, "gpsweek"), 2210.0)

    def test_header_str(self):
        self.assertEqual(
            getattr(self.z3d_obj, "header_str"),
            b"\n\n\nGPS Brd339 Logfile\nVersion = 4147\nMain.hex Buildnum = 5357\nChannelSerial = 0xD474777C\nFpga Buildnum = 1125\nBox Serial = 0x0000010013A20040\nBox number = 24\nChannel = 5\nA/D Rate = 256\nA/D Gain =  1\nPeriod = 4294967295\nDuty = 32767\nDutyOff = 1\nDutyNormalized = inf\nLogTerminal = N\nTx.Freq = 0.000000\nTx.Duty = inf\nLat = 0.706816081\nLong = -2.038914402\nAlt = 1456.300\nNumSats = 17\nGpsWeek = 2210\nAttenChannelsMask = 0x80\nChannelGain = 1.0000\nCh.Factor = 9.536743164062e-10\n\x00                                    \r\n\x00",
        )

    def test_lat(self):
        self.assertEqual(getattr(self.z3d_obj, "lat"), 40.49757833327694)

    def test_logterminal(self):
        self.assertEqual(getattr(self.z3d_obj, "logterminal"), "N")

    def test_long(self):
        self.assertEqual(getattr(self.z3d_obj, "long"), -116.8211900230401)

    def test_main_hex_buildnum(self):
        self.assertEqual(getattr(self.z3d_obj, "main_hex_buildnum"), 5357.0)

    def test_numsats(self):
        self.assertEqual(getattr(self.z3d_obj, "numsats"), 17.0)

    def test_old_version(self):
        self.assertEqual(getattr(self.z3d_obj, "old_version"), False)

    def test_period(self):
        self.assertEqual(getattr(self.z3d_obj, "period"), 4294967295.0)

    def test_tx_duty(self):
        self.assertEqual(getattr(self.z3d_obj, "tx_duty"), np.inf)

    def test_tx_freq(self):
        self.assertEqual(getattr(self.z3d_obj, "tx_freq"), 0.0)

    def test_version(self):
        self.assertEqual(getattr(self.z3d_obj, "version"), 4147.0)


# =============================================================================
# run
# =============================================================================
if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Adopted from TimeSeries reader, making all attributes properties for easier
reading and testing.

Module to read and parse native Phoenix Geophysics data formats of the MTU-5C Family

This module implements Streamed readers for segmented-decimated continuus-decimated
and native sampling rate time series formats of the MTU-5C family.

:author: Jorge Torres-Solis

Revised 2022 by J. Peacock 
"""

# =============================================================================
# Imports
# =============================================================================

from datetime import datetime

from struct import unpack_from
import string

from mt_metadata.timeseries import Station, Run, Electric, Magnetic
from mt_metadata.utils.mttime import MTime

# =============================================================================
class Header:
    """

    The header is 128 bytes with a specific format.  This reads in the 128
    bytes and provides properties to read each attribute of the header in
    the correct way.

    """

    def __init__(self, **kwargs):
        self.report_hw_sat = False
        self.header_length = 128
        self.ad_plus_minus_range = 5.0  # differential voltage range that the A/D can measure (Board model dependent)
        self._header = None
        self._recording_id = None
        self._channel_id = None

        self.channel_map = {
            0: "hx",
            1: "hy",
            2: "hz",
            3: "ex",
            4: "ey",
            5: "h1",
            6: "h2",
            7: "h3",
        }

        for key, value in kwargs.items():
            setattr(self, key, value)
        self._unpack_dict = {
            "file_type": {"dtype": "B", "index": 0},
            "file_version": {"dtype": "B", "index": 1},
            "header_length": {"dtype": "H", "index": 2},
            "instrument_type": {"dtype": "8s", "index": 4},
            "instrument_serial_number": {"dtype": "cccccccc", "index": 12},
            "recording_id": {"dtype": "I", "index": 20},
            "channel_id": {"dtype": "B", "index": 24},
            "file_sequence": {"dtype": "I", "index": 25},
            "frag_period": {"dtype": "H", "index": 29},
            "ch_board_model": {"dtype": "8s", "index": 31},
            "ch_board_serial": {"dtype": "8s", "index": 39},
            "ch_firmware": {"dtype": "I", "index": 47},
            "hardware_configuration": {"dtype": "BBBBBBBB", "index": 51},
            "sample_rate_base": {"dtype": "H", "index": 59},
            "sample_rate_exp": {"dtype": "b", "index": 61},
            "bytes_per_sample": {"dtype": "B", "index": 62},
            "frame_size": {"dtype": "I", "index": 63},
            "decimation_node_id": {"dtype": "H", "index": 67},
            "frame_rollover_count": {"dtype": "H", "index": 69},
            "gps_long": {"dtype": "f", "index": 71},
            "gps_lat": {"dtype": "f", "index": 75},
            "gps_elevation": {"dtype": "f", "index": 79},
            "gps_horizontal_accuracy": {"dtype": "I", "index": 83},
            "gps_vertical_accuracy": {"dtype": "I", "index": 87},
            "timing_status": {"dtype": "BBH", "index": 91},
            "future1": {"dtype": "b", "index": 95},
            "future2": {"dtype": "i", "index": 97},
            "saturated_frames": {"dtype": "H", "index": 101},
            "missing_frames": {"dtype": "H", "index": 103},
            "battery_voltage_mv": {"dtype": "H", "index": 105},
            "min_signal": {"dtype": "f", "index": 107},
            "max_signal": {"dtype": "f", "index": 111},
        }

    def __str__(self):
        lines = [
            f"channel_id: {self.channel_id}   channel_type: {self.channel_type}"
        ]
        lines += ["-" * 40]
        for key in [
            "instrument_type",
            "instrument_serial_number",
            "gps_lat",
            "gps_long",
            "gps_elevation",
            "recording_start_time",
            "sample_rate",
            "saturated_frames",
            "missing_frames",
            "max_signal",
            "min_signal",
        ]:
            lines.append(f"\t{key:<25}: {getattr(self, key)}")
        return "\n".join(lines)

    def __repr__(self):
        return self.__str__()

    def _has_header(self):
        if self._header is not None:
            return True
        return False

    def _unpack_value(self, key):
        if self._has_header():
            return unpack_from(
                self._unpack_dict[key]["dtype"],
                self._header,
                self._unpack_dict[key]["index"],
            )

    @property
    def file_type(self):
        if self._has_header():
            return self._unpack_value("file_type")[0]

    @property
    def file_version(self):
        if self._has_header():
            return self._unpack_value("file_version")[0]

    @property
    def header_length(self):
        if self._has_header():
            self._header_length = self._unpack_value("header_length")[0]
        return self._header_length

    @header_length.setter
    def header_length(self, value):
        self._header_length = value

    @property
    def instrument_type(self):
        if self._has_header():
            return (
                self._unpack_value("instrument_type")[0]
                .decode("utf-8")
                .strip(" ")
                .strip("\x00")
            )

    @property
    def instrument_serial_number(self):
        if self._has_header():
            return (
                b"".join(self._unpack_value("instrument_serial_number"))
                .strip(b"\x00")
                .decode("utf-8")
            )

    @property
    def recording_id(self):
        if self._recording_id is None:
            if self._has_header():
                return self._unpack_value("recording_id")[0]
        else:
            return self._recording_id

    @recording_id.setter
    def recording_id(self, value):
        if isinstance(value, str):
            value = int(value, 16)
        self._recording_id = value

    @property
    def recording_start_time(self):
        """
        The actual data recording starts 1 second after the set start time.
        This is cause by the data logger starting up and initializing filter.
        This is taken care of in the segment start time

        See https://github.com/kujaku11/PhoenixGeoPy/tree/main/Docs for more
        information.

        :return: DESCRIPTION
        :rtype: TYPE

        """

        return MTime(datetime.fromtimestamp(self.recording_id))

    @property
    def channel_id(self):
        if self._channel_id is None:
            if self._has_header():
                return int(self._unpack_value("channel_id")[0])
        else:
            return self._channel_id

    @channel_id.setter
    def channel_id(self, value):
        self._channel_id = int(value)

    @property
    def file_sequence(self):
        if self._has_header():
            return self._unpack_value("file_sequence")[0]

    @property
    def frag_period(self):
        if self._has_header():
            return self._unpack_value("frag_period")[0]

    @property
    def ch_board_model(self):
        if self._has_header():
            return (
                self._unpack_value("ch_board_model")[0]
                .decode("utf-8")
                .strip(" ")
            )

    @property
    def board_model_main(self):
        if self._has_header():
            return self.ch_board_model[0:5]

    @property
    def board_model_revision(self):
        if self._has_header():
            return self.ch_board_model[6:1]

    @property
    def ch_board_serial(self):
        if self._has_header():
            value = (
                self._unpack_value("ch_board_serial")[0]
                .decode("utf-8")
                .strip("\x00")
            )
            # handle the case of backend < v0.14, which puts '--------' in ch_ser
            if all(chars in string.hexdigits for chars in value):
                return int(value, 16)
            else:
                return 0

    @property
    def ch_firmware(self):
        if self._has_header():
            return self._unpack_value("ch_firmware")[0]

    @property
    def hardware_configuration(self):
        if self._has_header():
            return self._unpack_value("hardware_configuration")

    @property
    def channel_type(self):
        if self._has_header():
            if self.hardware_configuration[1] & 0x08 == 0x08:
                return "E"
            else:
                return "H"

    @property
    def detected_channel_type(self):
        # Channel type detected by electronics
        # this normally matches self.channel_type, but used in electronics design and testing
        if self._has_header():
            if self.hardware_configuration[1] & 0x20 == 0x20:
                return "E"
            else:
                return "H"

    @property
    def lp_frequency(self):
        if self._has_header():
            # LPF on
            if self.hardware_configuration[0] & 0x80 == 0x80:
                if self.hardware_configuration[0] & 0x03 == 0x03:
                    return 10
                elif self.hardware_configuration[0] & 0x03 == 0x02:
                    if (
                        self.board_model_main == "BCM03"
                        or self.board_model_main == "BCM06"
                    ):
                        return 1000
                    else:
                        return 100
                elif self.hardware_configuration[0] & 0x03 == 0x01:
                    if (
                        self.board_model_main == "BCM03"
                        or self.board_model_main == "BCM06"
                    ):
                        return 10000
                    else:
                        return 1000
            # LPF off
            else:
                if (
                    self.board_model_main == "BCM03"
                    or self.board_model_main == "BCM06"
                ):
                    return 17800
                else:
                    return 10000

    @property
    def preamp_gain(self):
        preamp_gain = 1.0
        if self._has_header():
            if self.channel_type == "?":
                raise Exception(
                    "Channel type must be set before attemting to calculate preamp gain"
                )
            preamp_on = bool(self.hardware_configuration[0] & 0x10)
            if self.channel_type == "E":
                if preamp_on:
                    if (
                        self.board_model_main == "BCM01"
                        or self.board_model_main == "BCM03"
                    ):
                        preamp_gain = 4.0
                        if self.board_model_revision == "L":
                            # Account for BCM01-L experimental prototype
                            preamp_gain = 8.0
                    else:
                        preamp_gain = 8.0
                        # Acount for experimental prototype BCM05-A
                        if self.ch_board_model[0:7] == "BCM05-A":
                            preamp_gain = 4.0
        return preamp_gain

    @property
    def channel_main_gain(self):
        # The value of the main gain of the board
        main_gain = 1
        if self._has_header():
            # BCM05-B and BCM06 introduced different selectable gains
            new_gains = (
                True  # we asume any newer board will have the new gain banks
            )
            if (
                self.board_model_main == "BCM01"
                or self.board_model_main == "BCM03"
            ):
                # Original style 24 KSps boards and original 96 KSps boards
                new_gains = False
            if self.ch_board_model[0:7] == "BCM05-A":
                # Acount for experimental prototype BCM05-A, which also had original gain banks
                new_gains = False
            if self.hardware_configuration[0] & 0x0C == 0x00:
                main_gain = 1.0
            elif self.hardware_configuration[0] & 0x0C == 0x04:
                main_gain = 4.0
            elif self.hardware_configuration[0] & 0x0C == 0x08:
                main_gain = 6.0
                if not new_gains:
                    main_gain = 16.0
            elif self.hardware_configuration[0] & 0x0C == 0x0C:
                main_gain = 8.0
                if not new_gains:
                    main_gain = 32.0
        return main_gain

    @property
    def intrinsic_circuitry_gain(self):
        """
        This function will adjust the intrinsic circuitry gain based on the
        sensor range configuration in the configuration fingerprint

        For this, we consider that for the Electric channel, calibration path, or H-legacy
        sensors all go through a 1/4 gain stage, and then they get a virtial x2 gain from
        Single-ended-diff before the A/D. In the case of newer sensors (differential)
        instead of a 1/4 gain stage, there is only a 1/2 gain stage

        Therefore, in the E,cal and legacy sensor case the circuitry gain is 1/2, while for
        newer sensors it is 1

        .. note:: Circuitry Gain not directly configurable by the user

        """

        intrinsic_circuitry_gain = 0.5
        if self._has_header():
            if self.channel_type == "?":
                raise Exception(
                    "Channel type must be set before attemting to calculate preamp gain"
                )
            intrinsic_circuitry_gain = 0.5
            if self.channel_type == "H":
                if self.hardware_configuration[1] & 0x01 == 0x01:
                    intrinsic_circuitry_gain = 1.0
        return intrinsic_circuitry_gain

    @property
    def attenuator_gain(self):
        # Asume attenuator off
        attenuator_gain = 1.0
        if self._has_header():
            if self.channel_type == "?":
                raise Exception(
                    "Channel type must be set before attemting to calculate preamp gain"
                )
            attenuator_on = bool(self.hardware_configuration[4] & 0x01)
            if attenuator_on and self.channel_type == "E":
                new_attenuator = True  # By default assume that we are dealing with a newer types of boards
                if (
                    self.board_model_main == "BCM01"
                    or self.board_model_main == "BCM03"
                ):
                    # Original style 24 KSps boards and original 96 KSps boards
                    new_attenuator = False
                if self.ch_board_model[0:7] == "BCM05-A":
                    # Acount for experimental prototype BCM05-A, which also had original gain banks
                    new_attenuator = False
                if new_attenuator:
                    attenuator_gain = 523.0 / 5223.0
                else:
                    attenuator_gain = 0.1
        return attenuator_gain

    # Board-wide gains
    @property
    def total_selectable_gain(self):
        # Total of the gain that is selectable by the user (i.e. att * pre * gain)
        if self._has_header():
            return (
                self.channel_main_gain * self.preamp_gain * self.attenuator_gain
            )
        return 1.0

    @property
    def total_circuitry_gain(self):
        # Total board Gain both intrinsic gain and user-seletable gain in circuit
        if self._has_header():
            return self.total_selectable_gain * self.intrinsic_circuitry_gain
        return 0.5

    @property
    def sample_rate_base(self):
        if self._has_header():
            return self._unpack_value("sample_rate_base")[0]

    @property
    def sample_rate_exp(self):
        if self._has_header():
            return self._unpack_value("sample_rate_exp")[0]

    @property
    def sample_rate(self):
        if self._has_header():
            if self.sample_rate_exp != 0:
                return self.sample_rate_base * pow(10, self.sample_rate_exp)
            return self.sample_rate_base

    @property
    def bytes_per_sample(self):
        if self._has_header():
            return self._unpack_value("bytes_per_sample")[0]

    @property
    def frame_size(self):
        if self._has_header():
            return self._unpack_value("frame_size")[0]

    @property
    def data_footer(self):
        if self._has_header():
            return self.frame_size >> 24

    @property
    def frame_size_bytes(self):
        if self._has_header():
            return self.frame_size & 0x0FFFFFF

    @property
    def decimation_node_id(self):
        if self._has_header():
            return self._unpack_value("decimation_node_id")[0]

    @property
    def frame_rollover_count(self):
        if self._has_header():
            return self._unpack_value("frame_rollover_count")[0]

    @property
    def gps_long(self):
        if self._has_header():
            return self._unpack_value("gps_long")[0]

    @property
    def gps_lat(self):
        if self._has_header():
            return self._unpack_value("gps_lat")[0]

    @property
    def gps_elevation(self):
        if self._has_header():
            return self._unpack_value("gps_elevation")[0]

    @property
    def gps_horizontal_accuracy(self):
        """
        In millimeters

        """
        if self._has_header():
            return self._unpack_value("gps_horizontal_accuracy")[0] / 1000

    @property
    def gps_vertical_accuracy(self):
        """
        In millimeters

        """
        if self._has_header():
            return self._unpack_value("gps_vertical_accuracy")[0] / 1000

    @property
    def timing_status(self):
        if self._has_header():
            return self._unpack_value("timing_status")

    @property
    def timing_flags(self):
        if self._has_header():
            return self.timing_status[0]

    @property
    def timing_sat_count(self):
        if self._has_header():
            return self.timing_status[1]

    @property
    def timing_stability(self):
        if self._has_header():
            return self.timing_status[2]

    @property
    def future1(self):
        if self._has_header():
            return self._unpack_value("future1")[0]

    @property
    def future2(self):
        if self._has_header():
            return self._unpack_value("future2")[0]

    @property
    def saturated_frames(self):
        if self._has_header():
            value = self._unpack_value("saturated_frames")[0]
            if value & 0x80 == 0x80:
                value &= 0x7F
                value <<= 4
            return value

    @property
    def missing_frames(self):
        if self._has_header():
            return self._unpack_value("missing_frames")[0]

    @property
    def battery_voltage_v(self):
        if self._has_header():
            return self._unpack_value("battery_voltage_mv")[0] / 1000

    @property
    def min_signal(self):
        if self._has_header():
            return self._unpack_value("min_signal")[0]

    @property
    def max_signal(self):
        if self._has_header():
            return self._unpack_value("max_signal")[0]

    def unpack_header(self, stream):
        if self.header_length > 0:
            # be sure to read from the beginning of the file
            stream.seek(0)
            self._header = stream.read(self.header_length)
        else:
            return

    def channel_metadata(self):
        """
        translate metadata to channel metadata
        :return: DESCRIPTION
        :rtype: TYPE

        """

        if self.channel_type.lower() in ["h"]:
            ch = Magnetic()
        elif self.channel_type.lower() in ["e"]:
            ch = Electric()
        try:
            ch.component = self.channel_map[self.channel_id]
        except KeyError:
            print(f"Could not find {self.channel_id} in channel_map")
        ch.channel_number = self.channel_id
        ch.time_period.start = self.recording_start_time
        ch.sample_rate = self.sample_rate

        return ch

    def run_metadata(self):
        """
        translate to run metadata

        :return: DESCRIPTION
        :rtype: TYPE

        """

        r = Run()
        r.data_logger.type = self.instrument_type
        r.data_logger.id = self.instrument_serial_number
        r.data_logger.manufacturer = "Phoenix Geophysics"
        r.data_logger.timing_system.uncertainty = self.timing_stability
        r.sample_rate = self.sample_rate
        r.data_logger.power_source.voltage.start = self.battery_voltage_v
        r.channels.append(self.channel_metadata())

        return r

    def station_metadata(self):
        """
        translate to station metadata

        """

        s = Station()
        s.location.latitude = self.gps_lat
        s.location.longitude = self.gps_long
        s.location.elevation = self.gps_elevation
        s.runs.append(self.run_metadata())

        return s



# -*- coding: utf-8 -*-

# =============================================================================
# Imports
# =============================================================================
import unittest

import numpy as np
from mth5.timeseries import ChannelTS, RunTS
from mt_metadata.timeseries import Electric, Magnetic, Auxiliary, Run, Station

# =============================================================================


class TestMakeRunTS(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.maxDiff = None
        channel_list = []
        self.common_start = "2020-01-01T00:00:00+00:00"
        self.sample_rate = 1.0
        self.n_samples = 4096
        t = np.arange(self.n_samples)
        data = np.sum(
            [
                np.cos(2 * np.pi * w * t + phi)
                for w, phi in zip(np.logspace(-3, 3, 20), np.random.rand(20))
            ],
            axis=0,
        )

        self.station_metadata = Station(id="mt001")
        self.run_metadata = Run(id="001")

        for component in ["hx", "hy", "hz"]:
            h_metadata = Magnetic(component=component)
            h_metadata.time_period.start = self.common_start
            h_metadata.sample_rate = self.sample_rate
            h_channel = ChannelTS(
                channel_type="magnetic",
                data=data,
                channel_metadata=h_metadata,
                run_metadata=self.run_metadata,
                station_metadata=self.station_metadata,
            )
            channel_list.append(h_channel)

        for component in ["ex", "ey"]:
            e_metadata = Electric(component=component)
            e_metadata.time_period.start = self.common_start
            e_metadata.sample_rate = self.sample_rate
            e_channel = ChannelTS(
                channel_type="electric",
                data=data,
                channel_metadata=e_metadata,
                run_metadata=self.run_metadata,
                station_metadata=self.station_metadata,
            )
            channel_list.append(e_channel)

        aux_metadata = Auxiliary(component="temperature")
        aux_metadata.time_period.start = self.common_start
        aux_metadata.sample_rate = self.sample_rate
        aux_channel = ChannelTS(
            channel_type="auxiliary",
            data=np.random.rand(self.n_samples) * 30,
            channel_metadata=aux_metadata,
            run_metadata=self.run_metadata,
            station_metadata=self.station_metadata,
        )
        channel_list.append(aux_channel)

        self.run_ts = RunTS(channel_list)

    def test_station_metadata(self):
        with self.subTest("station id"):
            self.assertEqual(
                self.run_ts.station_metadata.id, self.station_metadata.id
            )
        with self.subTest("start"):
            self.assertEqual(
                self.run_ts.station_metadata.time_period.start,
                self.common_start,
            )
        with self.subTest("run list"):
            self.assertListEqual(
                self.run_ts.station_metadata.run_list,
                [self.run_metadata.id],
            )
        with self.subTest("channels_recorded"):
            self.assertListEqual(
                ["ex", "ey", "hx", "hy", "hz", "temperature"],
                self.run_ts.station_metadata.channels_recorded,
            )

    def test_run_metadata(self):
        with self.subTest("run id"):
            self.assertEqual(self.run_ts.run_metadata.id, self.run_metadata.id)
        with self.subTest("start"):
            self.assertEqual(
                self.run_ts.run_metadata.time_period.start,
                self.common_start,
            )
        with self.subTest("channels"):
            self.assertListEqual(
                self.run_ts.run_metadata.channels_recorded_all,
                ["hx", "hy", "hz", "ex", "ey", "temperature"],
            )

    def test_channels(self):
        for comp in self.run_ts.channels:
            ch = getattr(self.run_ts, comp)
            with self.subTest("start"):
                self.assertEqual(
                    ch.channel_metadata.time_period.start,
                    self.common_start,
                )
            with self.subTest("sample rate"):
                self.assertEqual(
                    ch.sample_rate,
                    self.sample_rate,
                )
            with self.subTest("n samples"):
                self.assertEqual(
                    ch.n_samples,
                    4096,
                )


# =============================================================================
# run
# =============================================================================
if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Created on Wed Jun  9 08:55:16 2021

:copyright: 
    Jared Peacock (jpeacock@usgs.gov)

:license: MIT

"""

# =============================================================================
# Imports
# =============================================================================
import numpy as np

from mt_metadata.timeseries.filters import FrequencyResponseTableFilter

from mth5.groups.base import BaseGroup

# =============================================================================
# fap Group
# =============================================================================
class FAPGroup(BaseGroup):
    """
    Container for fap type filters

    """

    def __init__(self, group, **kwargs):
        super().__init__(group, **kwargs)

    @property
    def filter_dict(self):
        """

        Dictionary of available fap filters

        :return: DESCRIPTION
        :rtype: TYPE

        """
        f_dict = {}
        for key in self.hdf5_group.keys():
            fap_group = self.hdf5_group[key]
            f_dict[key] = {
                "type": fap_group.attrs["type"],
                "hdf5_ref": fap_group.ref,
            }

        return f_dict

    def add_filter(self, name, frequency, amplitude, phase, fap_metadata):
        """

        create an HDF5 group/dataset from information given.
        
        :param name: name of the filter
        :type name: string
        :param frequency: frequency array in samples per second
        :type frequency: list, np.ndarray
        :param amplitude: amplitude array in units of units out
        :type amplitude: list, np.ndarray
        :param phase: Phase in degrees
        :type phase: list, np.ndarray
        :param fap_metadata: other metadata for the filter see
        :class:`mt_metadata.timeseries.filters.FrequencyResponseTableFilter` \
         for details on entries
        :type fap_metadata: dictionary
        :return: DESCRIPTION
        :rtype: TYPE

        """

        # create a group for the filter by the name
        fap_filter_group = self.hdf5_group.create_group(name)

        # create datasets for the poles and zeros
        fap_ds = fap_filter_group.create_dataset(
            "fap_table",
            frequency.shape,
            dtype=np.dtype(
                [("frequency", float), ("amplitude", float), ("phase", float)]
            ),
            **self.dataset_options,
        )

        fap_ds[:] = [(f, a, p) for f, a, p in zip(frequency, amplitude, phase)]

        # fill in the metadata
        fap_filter_group.attrs.update(fap_metadata)

        return fap_filter_group

    def remove_filter(self):
        pass

    def get_filter(self, name):
        """
        Get a filter from the name

        :param name: name of the filter
        :type name: string

        :return: HDF5 group of the fap filter
        """
        return self.hdf5_group[name]

    def from_object(self, fap_object):
        """
        make a filter from a :class:`mt_metadata.timeseries.filters.PoleZeroFilter`

        :param fap_object: MT metadata PoleZeroFilter
        :type fap_object: :class:`mt_metadata.timeseries.filters.PoleZeroFilter`

        """

        if not isinstance(fap_object, FrequencyResponseTableFilter):
            msg = f"Filter must be a FrequencyResponseTableFilter not {type(fap_object)}"
            self.logger.error(msg)
            raise TypeError(msg)

        input_dict = fap_object.to_dict(single=True, required=False)
        input_dict.pop("frequencies")
        input_dict.pop("amplitudes")
        input_dict.pop("phases")
        for k, v in input_dict.items():
            if v is None:
                input_dict[k] = str(v)

        fap_group = self.add_filter(
            fap_object.name,
            fap_object.frequencies,
            fap_object.amplitudes,
            fap_object.phases,
            input_dict,
        )
        return fap_group

    def to_object(self, name):
        """
        make a :class:`mt_metadata.timeseries.filters.pole_zeros_filter` object

        :return: DESCRIPTION
        :rtype: TYPE

        """

        fap_group = self.get_filter(name)

        fap_obj = FrequencyResponseTableFilter(**fap_group.attrs)

        try:
            fap_obj.frequencies = fap_group["fap_table"]["frequency"][:]
        except TypeError:
            self.logger.debug(f"fap filter {name} has no frequency")
            fap_obj.frequencies = []

        try:
            fap_obj.amplitudes = fap_group["fap_table"]["amplitude"][:]
        except TypeError:
            self.logger.debug(f"fap filter {name} has no amplitudes")
            fap_obj.amplitudes = []

        try:
            fap_obj.phases = fap_group["fap_table"]["phase"][:]
        except TypeError:
            self.logger.debug(f"fap filter {name} has no phases")
            fap_obj.phases = []

        return fap_obj



# -*- coding: utf-8 -*-
"""
Created on Fri Aug 19 16:39:30 2022

@author: jpeacock
"""

# =============================================================================
# Imports
# =============================================================================
import unittest
from pathlib import Path
from collections import OrderedDict


from mth5.io.nims import NIMS, read_nims
from mt_metadata.utils.mttime import MTime

# =============================================================================


@unittest.skipIf(
    "peacock" not in str(Path(__file__).as_posix()),
    "Big data on local machine",
)
class TestReadNIMS(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.nims_obj = NIMS(
            r"c:\Users\jpeacock\OneDrive - DOI\mt\nims\mnp300a.BIN"
        )
        self.nims_obj.read_nims()
        self.maxDiff = None

    def test_site_name(self):
        self.assertEqual(self.nims_obj.site_name, "Budwieser Spring")

    def test_state_province(self):
        self.assertEqual(self.nims_obj.state_province, "CA")

    def test_country(self):
        self.assertEqual(self.nims_obj.country, "USA")

    def test_box_id(self):
        self.assertEqual(self.nims_obj.box_id, "1105-3")

    def test_mag_id(self):
        self.assertEqual(self.nims_obj.mag_id, "1305-3")

    def test_ex_length(self):
        self.assertEqual(self.nims_obj.ex_length, 109.0)

    def test_ex_azimuth(self):
        self.assertEqual(self.nims_obj.ex_azimuth, 0.0)

    def test_ey_length(self):
        self.assertEqual(self.nims_obj.ey_length, 101.0)

    def test_ey_azimuth(self):
        self.assertEqual(self.nims_obj.ey_azimuth, 90.0)

    def test_n_electrode_id(self):
        self.assertEqual(self.nims_obj.n_electrode_id, "1")

    def test_s_electrode_id(self):
        self.assertEqual(self.nims_obj.s_electrode_id, "2")

    def test_e_electrode_id(self):
        self.assertEqual(self.nims_obj.e_electrode_id, "3")

    def test_w_electrode_id(self):
        self.assertEqual(self.nims_obj.w_electrode_id, "4")

    def test_ground_electrode_info(self):
        self.assertEqual(self.nims_obj.ground_electrode_info, None)

    def test_header_gps_stamp(self):
        self.assertEqual(
            self.nims_obj.header_gps_stamp, MTime("2019-09-26 18:29:29")
        )

    def test_header_gps_latitude(self):
        self.assertEqual(self.nims_obj.header_gps_latitude, 34.7268)

    def test_header_gps_longitude(self):
        self.assertEqual(self.nims_obj.header_gps_longitude, -115.735)

    def test_header_gps_elevation(self):
        self.assertEqual(self.nims_obj.header_gps_elevation, 939.8)

    def test_operator(self):
        self.assertEqual(self.nims_obj.operator, "KP")

    def test_comments(self):
        self.assertEqual(
            self.nims_obj.comments,
            (
                "N/S: CRs: .764/.769 DCV: 3.5 ACV:1 \n"
                "E/W:  CRs: .930/.780 DCV: 28.2 ACV: 1\n"
                "Hwy 40 800m S, X array"
            ),
        )

    def test_run_id(self):
        self.assertEqual(self.nims_obj.run_id, "mnp300a")

    def test_data_start_seek(self):
        self.assertEqual(self.nims_obj.data_start_seek, 946)

    def test_block_size(self):
        self.assertEqual(self.nims_obj.block_size, 131)

    def test_block_sequence(self):
        self.assertEqual(self.nims_obj.block_sequence, [1, 131])

    def test_sample_rate(self):
        self.assertEqual(self.nims_obj.sample_rate, 8)

    def test_e_conversion_factor(self):
        self.assertEqual(
            self.nims_obj.e_conversion_factor, 2.44141221047903e-06
        )

    def test_h_conversion_factor(self):
        self.assertEqual(self.nims_obj.h_conversion_factor, 0.01)

    def test_t_conversion_factor(self):
        self.assertEqual(self.nims_obj.t_conversion_factor, 70)

    def test_t_offset(self):
        self.assertEqual(self.nims_obj.t_offset, 18048)

    def test_ground_electrodeinfo(self):
        self.assertEqual(self.nims_obj.ground_electrodeinfo, "Cu")

    def test_data_shape(self):
        self.assertEqual(self.nims_obj.ts_data.shape, (3357016, 5))

    def test_stamps_size(self):
        self.assertEqual(len(self.nims_obj.stamps), 402)

    def test_start(self):
        self.assertEqual(
            self.nims_obj.start_time, MTime("2019-09-26T18:33:21+00:00")
        )

    def test_end(self):
        self.assertEqual(
            self.nims_obj.end_time, MTime("2019-10-01T15:07:07.875000+00:00")
        )

    def test_latitude(self):
        self.assertAlmostEqual(self.nims_obj.latitude, 34.726826667)

    def test_longitude(self):
        self.assertAlmostEqual(self.nims_obj.longitude, -115.73501166)

    def test_elevation(self):
        self.assertAlmostEqual(self.nims_obj.elevation, 940.4)

    def test_declination(self):
        self.assertEqual(self.nims_obj.declination, 13.1)


@unittest.skipIf(
    "peacock" not in str(Path(__file__).as_posix()),
    "Big data on local machine",
)
class TestNIMSToRunTS(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.runts = read_nims(
            r"c:\Users\jpeacock\OneDrive - DOI\mt\nims\mnp300a.BIN"
        )
        self.maxDiff = None

    def test_station_metadata(self):
        station_metadata = OrderedDict(
            [
                ("acquired_by.name", None),
                (
                    "channels_recorded",
                    ["ex", "ey", "hx", "hy", "hz", "temperature"],
                ),
                ("data_type", "BBMT"),
                ("geographic_name", "Budwieser Spring, CA, USA"),
                ("id", "mnp300"),
                ("location.declination.model", "WMM"),
                ("location.declination.value", 13.1),
                ("location.elevation", 940.4),
                ("location.latitude", 34.72682666666667),
                ("location.longitude", -115.73501166666667),
                ("orientation.method", None),
                ("orientation.reference_frame", "geomagnetic"),
                ("provenance.creation_time", "1980-01-01T00:00:00+00:00"),
                ("provenance.software.author", None),
                ("provenance.software.name", None),
                ("provenance.software.version", None),
                ("provenance.submitter.email", None),
                ("provenance.submitter.organization", None),
                ("release_license", "CC0-1.0"),
                ("run_list", ["mnp300a"]),
                ("time_period.end", "2019-10-01T15:07:07.875000+00:00"),
                ("time_period.start", "2019-09-26T18:33:21+00:00"),
            ]
        )

        self.assertDictEqual(
            self.runts.station_metadata.to_dict(single=True), station_metadata
        )

    def test_run_metadata(self):
        run_metadata = OrderedDict(
            [
                ("channels_recorded_auxiliary", ["temperature"]),
                ("channels_recorded_electric", ["ex", "ey"]),
                ("channels_recorded_magnetic", ["hx", "hy", "hz"]),
                (
                    "comments",
                    "N/S: CRs: .764/.769 DCV: 3.5 ACV:1 \nE/W:  CRs: .930/.780 DCV: 28.2 ACV: 1\nHwy 40 800m S, X array",
                ),
                ("data_logger.firmware.author", "B. Narod"),
                ("data_logger.firmware.name", "nims"),
                ("data_logger.firmware.version", "1.0"),
                ("data_logger.id", "1105-3"),
                ("data_logger.manufacturer", "Narod"),
                ("data_logger.model", "1105-3"),
                ("data_logger.timing_system.drift", 0.0),
                ("data_logger.timing_system.type", "GPS"),
                ("data_logger.timing_system.uncertainty", 0.0),
                ("data_logger.type", "long period"),
                ("data_type", "MTLP"),
                ("id", "mnp300a"),
                ("sample_rate", 8.0),
                ("time_period.end", "2019-10-01T15:07:07.875000+00:00"),
                ("time_period.start", "2019-09-26T18:33:21+00:00"),
            ]
        )

        self.assertDictEqual(
            self.runts.run_metadata.to_dict(single=True), run_metadata
        )

    def test_ex_metadata(self):
        ex_metadata = OrderedDict(
            [
                ("channel_number", 4),
                ("component", "ex"),
                ("data_quality.rating.value", 0),
                ("dipole_length", 109.0),
                ("filter.applied", [False, False, False, False, False, False]),
                (
                    "filter.name",
                    [
                        "to_mt_units",
                        "dipole_109.00",
                        "nims_5_pole_butterworth",
                        "nims_1_pole_butterworth",
                        "e_analog_to_digital",
                        "ex_time_offset",
                    ],
                ),
                ("measurement_azimuth", 0.0),
                ("measurement_tilt", 0.0),
                ("negative.elevation", 0.0),
                ("negative.id", "2"),
                ("negative.latitude", 0.0),
                ("negative.longitude", 0.0),
                ("negative.manufacturer", None),
                ("negative.type", None),
                ("positive.elevation", 0.0),
                ("positive.id", "1"),
                ("positive.latitude", 0.0),
                ("positive.longitude", 0.0),
                ("positive.manufacturer", None),
                ("positive.type", None),
                ("sample_rate", 8.0),
                ("time_period.end", "2019-10-01T15:07:07.875000+00:00"),
                ("time_period.start", "2019-09-26T18:33:21+00:00"),
                ("type", "electric"),
                ("units", "counts"),
            ]
        )

        self.assertDictEqual(
            self.runts.ex.channel_metadata.to_dict(single=True), ex_metadata
        )

    def test_ey_metadata(self):
        ey_metadata = OrderedDict(
            [
                ("channel_number", 5),
                ("component", "ey"),
                ("data_quality.rating.value", 0),
                ("dipole_length", 101.0),
                ("filter.applied", [False, False, False, False, False, False]),
                (
                    "filter.name",
                    [
                        "to_mt_units",
                        "dipole_101.00",
                        "nims_5_pole_butterworth",
                        "nims_1_pole_butterworth",
                        "e_analog_to_digital",
                        "ey_time_offset",
                    ],
                ),
                ("measurement_azimuth", 90.0),
                ("measurement_tilt", 0.0),
                ("negative.elevation", 0.0),
                ("negative.id", "4"),
                ("negative.latitude", 0.0),
                ("negative.longitude", 0.0),
                ("negative.manufacturer", None),
                ("negative.type", None),
                ("positive.elevation", 0.0),
                ("positive.id", "3"),
                ("positive.latitude", 0.0),
                ("positive.longitude", 0.0),
                ("positive.manufacturer", None),
                ("positive.type", None),
                ("sample_rate", 8.0),
                ("time_period.end", "2019-10-01T15:07:07.875000+00:00"),
                ("time_period.start", "2019-09-26T18:33:21+00:00"),
                ("type", "electric"),
                ("units", "counts"),
            ]
        )

        self.assertDictEqual(
            self.runts.ey.channel_metadata.to_dict(single=True), ey_metadata
        )

    def test_hx_metadata(self):
        hx_metadata = OrderedDict(
            [
                ("channel_number", 1),
                ("component", "hx"),
                ("data_quality.rating.value", 0),
                ("filter.applied", [False, False, False]),
                (
                    "filter.name",
                    [
                        "nims_3_pole_butterworth",
                        "h_analog_to_digital",
                        "hx_time_offset",
                    ],
                ),
                ("location.elevation", 0.0),
                ("location.latitude", 0.0),
                ("location.longitude", 0.0),
                ("measurement_azimuth", 0.0),
                ("measurement_tilt", 0.0),
                ("sample_rate", 8.0),
                ("sensor.id", "1305-3"),
                ("sensor.manufacturer", "Barry Narod"),
                ("sensor.type", "fluxgate triaxial magnetometer"),
                ("time_period.end", "2019-10-01T15:07:07.875000+00:00"),
                ("time_period.start", "2019-09-26T18:33:21+00:00"),
                ("type", "magnetic"),
                ("units", "counts"),
            ]
        )

        self.assertDictEqual(
            self.runts.hx.channel_metadata.to_dict(single=True), hx_metadata
        )

    def test_hy_metadata(self):
        hy_metadata = OrderedDict(
            [
                ("channel_number", 2),
                ("component", "hy"),
                ("data_quality.rating.value", 0),
                ("filter.applied", [False, False, False]),
                (
                    "filter.name",
                    [
                        "nims_3_pole_butterworth",
                        "h_analog_to_digital",
                        "hy_time_offset",
                    ],
                ),
                ("location.elevation", 0.0),
                ("location.latitude", 0.0),
                ("location.longitude", 0.0),
                ("measurement_azimuth", 90.0),
                ("measurement_tilt", 0.0),
                ("sample_rate", 8.0),
                ("sensor.id", "1305-3"),
                ("sensor.manufacturer", "Barry Narod"),
                ("sensor.type", "fluxgate triaxial magnetometer"),
                ("time_period.end", "2019-10-01T15:07:07.875000+00:00"),
                ("time_period.start", "2019-09-26T18:33:21+00:00"),
                ("type", "magnetic"),
                ("units", "counts"),
            ]
        )

        self.assertDictEqual(
            self.runts.hy.channel_metadata.to_dict(single=True), hy_metadata
        )

    def test_hz_metadata(self):
        hz_metadata = OrderedDict(
            [
                ("channel_number", 3),
                ("component", "hz"),
                ("data_quality.rating.value", 0),
                ("filter.applied", [False, False, False]),
                (
                    "filter.name",
                    [
                        "nims_3_pole_butterworth",
                        "h_analog_to_digital",
                        "hz_time_offset",
                    ],
                ),
                ("location.elevation", 0.0),
                ("location.latitude", 0.0),
                ("location.longitude", 0.0),
                ("measurement_azimuth", 0.0),
                ("measurement_tilt", 0.0),
                ("sample_rate", 8.0),
                ("sensor.id", "1305-3"),
                ("sensor.manufacturer", "Barry Narod"),
                ("sensor.type", "fluxgate triaxial magnetometer"),
                ("time_period.end", "2019-10-01T15:07:07.875000+00:00"),
                ("time_period.start", "2019-09-26T18:33:21+00:00"),
                ("type", "magnetic"),
                ("units", "counts"),
            ]
        )

        self.assertDictEqual(
            self.runts.hz.channel_metadata.to_dict(single=True), hz_metadata
        )

    def test_temperature_metadata(self):
        t_metadata = OrderedDict(
            [
                ("channel_number", 6),
                ("component", "temperature"),
                ("data_quality.rating.value", 0),
                ("filter.applied", [False]),
                ("filter.name", []),
                ("location.elevation", 0.0),
                ("location.latitude", 0.0),
                ("location.longitude", 0.0),
                ("measurement_azimuth", 0.0),
                ("measurement_tilt", 0.0),
                ("sample_rate", 8.0),
                ("sensor.id", None),
                ("sensor.manufacturer", None),
                ("sensor.type", None),
                ("time_period.end", "2019-10-01T15:07:07.875000+00:00"),
                ("time_period.start", "2019-09-26T18:33:21+00:00"),
                ("type", "auxiliary"),
                ("units", "celsius"),
            ]
        )

        self.assertDictEqual(
            self.runts.temperature.channel_metadata.to_dict(single=True),
            t_metadata,
        )

    def test_dataset_dims(self):
        self.assertEqual(self.runts.dataset.dims["time"], 3357016)

    def test_calibrate(self):
        calibrated_run = self.runts.calibrate()

        for comp in ["hx", "hy", "hz"]:
            ch = getattr(calibrated_run, comp)
            with self.subTest("units"):
                self.assertEqual(ch.channel_metadata.units, "nT")
            with self.subTest("applied"):
                self.assertListEqual(
                    ch.channel_metadata.filter.applied, [True, True, True]
                )

        for comp in ["ex", "ey"]:
            ch = getattr(calibrated_run, comp)
            with self.subTest("units"):
                self.assertEqual(ch.channel_metadata.units, "mV/km")
            with self.subTest("applied"):
                self.assertListEqual(
                    ch.channel_metadata.filter.applied,
                    [True, True, True, True, True, True],
                )

        ch = getattr(calibrated_run, "temperature")
        with self.subTest("units"):
            self.assertEqual(ch.channel_metadata.units, "celsius")
        with self.subTest("applied"):
            self.assertListEqual(
                ch.channel_metadata.filter.applied,
                [True],
            )


# =============================================================================
# Run
# =============================================================================
if __name__ == "__main__":
    unittest.main()



# =============================================================================
# Imports
# =============================================================================
from pathlib import Path

from mth5.mth5 import MTH5
from mth5.helpers import close_open_files
from mth5.utils.mth5_logger import setup_logger

# =============================================================================

logger = setup_logger(__file__)


def initialize_mth5(h5_path, mode="a", file_version="0.1.0"):
    """
    mth5 initializer for the case of writting files.


    :param h5_path: path to file
    :type h5_path: string or pathlib.Path
    :param mode: how to open the file, options are

        - "r": read
        - "w": write
        - "a": append

    :type mode: string
    :return: mth5 object
    :rtype: :class:`mth5.MTH5`


    """
    h5_path = Path(h5_path)
    if mode == "w":
        if h5_path.exists():
            logger.warning("File exists, removing from file system.")
            close_open_files()
            h5_path.unlink()

    mth5_obj = MTH5(file_version=file_version)
    mth5_obj.open_mth5(str(h5_path), mode=mode)

    return mth5_obj


def read_back_data(
    mth5_path, station_id, run_id, survey=None, close_mth5=True, return_objects=[]
):
    """
    Testing helper function, used to confirm that the h5 file can be accessed
    and that the data size is as expected.

    :param mth5_path: the full path the the mth5 that this method is going to
     try to read
    :type mth5_path: Path or string
    :param station_id: the label for the station, e.g. "PKD"
    :type station_id: string
    :param run_id: The label for the run to read.  e.g. "001"
    :type run_id: string
    :param survey: The label for the survey associated with the run to read.
    :type survey: string
    :param close_mth5: Whether or not to close the mth5 object after reading
    :type close_mth5: bool
    :param return_objects: List of strings.  Specifies what, if anything to return.
    Allowed values: ["run", "run_ts"]
    :type return_objects: List of strings.
    :return: run object
    :rtype: :class:`mth5.groups.RunGroup`
    :return: run time series
    :rtype: :class:`mth5.timeseries.RunTS`

    """
    processing_config = {}
    processing_config["mth5_path"] = str(mth5_path)
    processing_config["local_station_id"] = station_id
    config = processing_config
    m = initialize_mth5(config["mth5_path"], mode="r")
    local_run_obj = m.get_run(config["local_station_id"], run_id, survey=survey)
    local_run_ts = local_run_obj.to_runts()
    data_array = local_run_ts.dataset.to_array()
    logger.info(f"data shape = {data_array.shape}")

    return_dict = {}
    if "run" in return_objects:
        return_dict["run"] = local_run_obj
    if "run_ts" in return_objects:
        return_dict["run_ts"] = local_run_ts
    if close_mth5:
        m.close_mth5()
    else:
        return_dict["mth5_obj"] = m
    return return_dict



# -*- coding: utf-8 -*-
"""
Created on Wed Dec 23 16:59:45 2020

:copyright:
    Jared Peacock (jpeacock@usgs.gov)

:license:
    MIT

"""

# =============================================================================
# Imports
# =============================================================================
from mth5.groups import BaseGroup, MasterSurveyGroup

# =============================================================================
# Experiment Group
# =============================================================================


class ExperimentGroup(BaseGroup):
    """
    Utility class to hold general information about the experiment and
    accompanying metadata for an MT experiment.

    To access the hdf5 group directly use `ExperimentGroup.hdf5_group`.

    >>> experiment = ExperimentGroup(hdf5_group)
    >>> experiment.hdf5_group.ref
    <HDF5 Group Reference>

    .. note:: All attributes should be input into the metadata object, that
             way all input will be validated against the metadata standards.
             If you change attributes in metadata object, you should run the
             `ExperimentGroup.write_metadata()` method.  This is a temporary
             solution, working on an automatic updater if metadata is changed.

    >>> experiment.metadata.existing_attribute = 'update_existing_attribute'
    >>> experiment.write_metadata()

    If you want to add a new attribute this should be done using the
    `metadata.add_base_attribute` method.

    >>> experiment.metadata.add_base_attribute('new_attribute',
    >>> ...                                'new_attribute_value',
    >>> ...                                {'type':str,
    >>> ...                                 'required':True,
    >>> ...                                 'style':'free form',
    >>> ...                                 'description': 'new attribute desc.',
    >>> ...                                 'units':None,
    >>> ...                                 'options':[],
    >>> ...                                 'alias':[],
    >>> ...                                 'example':'new attribute

    .. tip:: If you want ot add stations, reports, etc to the experiment this
              should be done from the MTH5 object.  This is to avoid
              duplication, at least for now.

    To look at what the structure of ``/Experiment`` looks like:

        >>> experiment
        /Experiment:
        ====================
            |- Group: Surveys
            -----------------
            |- Group: Reports
            -----------------
            |- Group: Standards
            -------------------
            |- Group: Stations
            ------------------

    """

    def __init__(self, group, **kwargs):

        super().__init__(group, **kwargs)

    @BaseGroup.metadata.getter
    def metadata(self):
        """Overwrite get metadata to include station information"""

        # need the try statement for when the file is initiated there is no
        # /Station group yet
        try:
            self._metadata.surveys = []
            for key in self.surveys_group.groups_list:
                key_group = self.surveys_group.get_survey(key)
                self._metadata.surveys.append(key_group.metadata)

        except KeyError:
            pass

        return self._metadata

    @property
    def surveys_group(self):
        return MasterSurveyGroup(self.hdf5_group["Surveys"])



# -*- coding: utf-8 -*-
"""
Created on Sat Sep 10 18:19:12 2022

@author: jpeacock
"""

# =============================================================================
# Imports
# =============================================================================
import unittest
from collections import OrderedDict
from pathlib import Path

import pandas as pd
import numpy as np

from mth5.io.phoenix import PhoenixCollection

# =============================================================================


@unittest.skipIf("peacock" not in str(Path(__file__).as_posix()), "local files")
class TestPhoenixCollection(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.pc = PhoenixCollection(
            r"c:\Users\jpeacock\OneDrive - DOI\mt\phoenix_example_data\10291_2019-09-06-015630"
        )

        self.df = self.pc.to_dataframe([150, 24000])
        self.runs = self.pc.get_runs([150, 24000])

        self.station = self.df.station.unique()[0]

    def test_file_path(self):
        self.assertIsInstance(self.pc.file_path, Path)

    def test_get_files(self):
        self.assertEqual(
            992,
            len(
                [
                    fn.name
                    for fn in self.pc.get_files(
                        self.pc._file_extension_map[150]
                    )
                ]
            ),
        )

    def test_df_columns(self):
        self.assertListEqual(
            self.pc._columns,
            self.df.columns.to_list(),
        )

    def test_df_shape(self):
        self.assertEqual(self.df.shape, (1984, 14))

    def test_df_types(self):
        self.df = self.pc._set_df_dtypes(self.df)
        with self.subTest("start"):
            self.assertTrue(
                self.df.start.dtype.type == pd._libs.tslibs.timestamps.Timestamp
            )
        with self.subTest("end"):
            self.assertTrue(
                self.df.end.dtype.type == pd._libs.tslibs.timestamps.Timestamp
            )

        with self.subTest("instrument_id"):
            self.assertTrue(self.df.instrument_id.dtype.type == np.object_)

        with self.subTest("calibration_fn"):
            self.assertTrue(self.df.calibration_fn.dtype.type == np.object_)

    def test_survey_id(self):
        self.assertTrue(
            (
                self.df.survey
                == list(self.pc.metadata_dict.values())[0].survey_metadata.id
            ).all()
        )

    def test_df_run_names_150(self):
        self.assertEqual(
            "sr150_0001", self.df[self.df.sample_rate == 150].run.unique()[0]
        )

    def test_df_run_names_24k(self):
        run_names = self.df[self.df.sample_rate == 24000].run.unique()

        with self.subTest("len"):
            self.assertEqual(124, run_names.size)

        with self.subTest("first"):
            self.assertEqual(run_names[0], "sr24k_0001")

        with self.subTest("last"):
            self.assertEqual(run_names[-1], "sr24k_0124")

    def test_runs_keys(self):
        self.assertEqual(125, len(self.runs[self.station].keys()))

    def test_run_dtype(self):
        self.assertIsInstance(self.runs, OrderedDict)

    def test_run_elements(self):
        for key, rdf in self.runs[self.station].items():
            with self.subTest(key):
                test_rdf = self.df[self.df.run == key]
                self.assertTrue(
                    (
                        self.df[self.df.run == key]
                        .iloc[0:8]
                        .eq(rdf)
                        .all(axis=0)
                        .all()
                    )
                )


# =============================================================================
# Run
# =============================================================================
if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
"""
Created on Thu May 13 13:45:27 2021

:copyright: 
    Jared Peacock (jpeacock@usgs.gov)

:license: MIT

"""
# =============================================================================
# Imports
# =============================================================================

import unittest
from pathlib import Path
import numpy as np

from mth5 import mth5, CHANNEL_DTYPE
from mth5.helpers import validate_name
from mt_metadata.timeseries import Experiment
from mt_metadata import MT_EXPERIMENT_SINGLE_STATION

fn_path = Path(__file__).parent
# =============================================================================
#
# =============================================================================
mth5.helpers.close_open_files()


class TestMTH5(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.maxDiff = None
        self.fn = fn_path.joinpath("test.mth5")
        self.mth5_obj = mth5.MTH5(file_version="0.2.0")
        self.mth5_obj.open_mth5(self.fn, mode="w")
        self.experiment = Experiment()
        self.experiment.from_xml(fn=MT_EXPERIMENT_SINGLE_STATION)
        self.survey_name = validate_name(self.experiment.surveys[0].id)
        self.mth5_obj.from_experiment(self.experiment)

    def test_surveys(self):
        survey = self.experiment.surveys[0]
        sg = self.mth5_obj.get_survey(self.survey_name)

        sd_md = survey.to_dict(single=True)
        sd_md.pop("hdf5_reference")
        sd_md.pop("mth5_type")

        h5_md = sg.metadata.to_dict(single=True)
        h5_md.pop("hdf5_reference")
        h5_md.pop("mth5_type")

        self.assertDictEqual(sd_md, h5_md)

    def test_stations(self):
        stations = self.experiment.surveys[0].stations
        for station in stations:
            with self.subTest(name=station.id):
                h5_station = self.mth5_obj.get_station(
                    station.id, self.survey_name
                )
                sd = station.to_dict(single=True)
                sd.pop("hdf5_reference")
                sd.pop("mth5_type")

                h5_sd = h5_station.metadata.to_dict(single=True)
                h5_sd.pop("hdf5_reference")
                h5_sd.pop("mth5_type")

                self.assertDictEqual(h5_sd, sd)

    def test_runs(self):
        runs = self.experiment.surveys[0].stations[0].runs
        for run in runs:
            with self.subTest(name=run.id):
                h5_run = self.mth5_obj.get_run(
                    self.experiment.surveys[0].stations[0].id,
                    run.id,
                    survey=self.survey_name,
                )
                sd = run.to_dict(single=True)
                sd.pop("hdf5_reference")
                sd.pop("mth5_type")

                h5_sd = h5_run.metadata.to_dict(single=True)
                h5_sd.pop("hdf5_reference")
                h5_sd.pop("mth5_type")

                self.assertDictEqual(h5_sd, sd)

    def test_to_run_ts(self):
        run_group = self.mth5_obj.get_run(
            self.experiment.surveys[0].stations[0].id,
            self.experiment.surveys[0].stations[0].runs[0].id,
            self.experiment.surveys[0].id,
        )
        run_ts = run_group.to_runts()

        for key in self.experiment.surveys[0].to_dict(single=True).keys():
            if key in ["hdf5_reference", "mth5_type"]:
                continue
            with self.subTest(f"survey.{key}"):
                self.assertEqual(
                    self.experiment.surveys[0].get_attr_from_name(key),
                    run_ts.survey_metadata.get_attr_from_name(key),
                )

        for key in (
            self.experiment.surveys[0].stations[0].to_dict(single=True).keys()
        ):
            if key in ["hdf5_reference", "mth5_type"]:
                continue

            with self.subTest(f"station.{key}"):
                if key in ["run_list"]:
                    self.assertListEqual(
                        ["a", "b", "c", "d", "e"],
                        run_ts.station_metadata.run_list,
                    )
                else:
                    self.assertEqual(
                        self.experiment.surveys[0]
                        .stations[0]
                        .get_attr_from_name(key),
                        run_ts.station_metadata.get_attr_from_name(key),
                    )

        for key in (
            self.experiment.surveys[0]
            .stations[0]
            .runs[0]
            .to_dict(single=True)
            .keys()
        ):
            if key in ["hdf5_reference", "mth5_type"]:
                continue
            with self.subTest(f"run.{key}"):
                if key in ["time_period.end"]:
                    self.assertNotEqual(
                        self.experiment.surveys[0]
                        .stations[0]
                        .runs[0]
                        .get_attr_from_name(key),
                        run_ts.run_metadata.get_attr_from_name(key),
                    )
                else:
                    self.assertEqual(
                        self.experiment.surveys[0]
                        .stations[0]
                        .runs[0]
                        .get_attr_from_name(key),
                        run_ts.run_metadata.get_attr_from_name(key),
                    )

    def test_to_channel_ts(self):
        channel_group = self.mth5_obj.get_channel(
            self.experiment.surveys[0].stations[0].id,
            self.experiment.surveys[0].stations[0].runs[0].id,
            self.experiment.surveys[0]
            .stations[0]
            .runs[0]
            .channels[0]
            .component,
            self.experiment.surveys[0].id,
        )
        ch_ts = channel_group.to_channel_ts()

        for key in self.experiment.surveys[0].to_dict(single=True).keys():
            if key in ["hdf5_reference", "mth5_type"]:
                continue
            with self.subTest(f"survey.{key}"):
                self.assertEqual(
                    self.experiment.surveys[0].get_attr_from_name(key),
                    ch_ts.survey_metadata.get_attr_from_name(key),
                )

        for key in (
            self.experiment.surveys[0].stations[0].to_dict(single=True).keys()
        ):
            if key in ["hdf5_reference", "mth5_type"]:
                continue

            with self.subTest(f"station.{key}"):
                if key in ["run_list", "channels_recorded"]:
                    self.assertListEqual(
                        ["a", "b", "c", "d", "e"],
                        ch_ts.station_metadata.run_list,
                    )
                else:
                    self.assertEqual(
                        self.experiment.surveys[0]
                        .stations[0]
                        .get_attr_from_name(key),
                        ch_ts.station_metadata.get_attr_from_name(key),
                    )

        for key in (
            self.experiment.surveys[0]
            .stations[0]
            .runs[0]
            .to_dict(single=True)
            .keys()
        ):
            if key in [
                "hdf5_reference",
                "mth5_type",
                "channels_recorded_magnetic",
                "channels_recorded_electric",
                "channels_recorded_auxiliary",
            ]:
                continue
            with self.subTest(f"run.{key}"):
                self.assertEqual(
                    self.experiment.surveys[0]
                    .stations[0]
                    .runs[0]
                    .get_attr_from_name(key),
                    ch_ts.run_metadata.get_attr_from_name(key),
                )

        for key in (
            self.experiment.surveys[0]
            .stations[0]
            .runs[0]
            .channels[0]
            .to_dict(single=True)
            .keys()
        ):
            if key in [
                "hdf5_reference",
                "mth5_type",
                "filter.name",
                "filter.applied",
            ]:
                continue
            with self.subTest(f"channel.{key}"):
                if key in ["time_period.end"]:
                    self.assertNotEqual(
                        self.experiment.surveys[0]
                        .stations[0]
                        .runs[0]
                        .channels[0]
                        .get_attr_from_name(key),
                        ch_ts.station_metadata.get_attr_from_name(key),
                    )
                else:
                    self.assertEqual(
                        self.experiment.surveys[0]
                        .stations[0]
                        .runs[0]
                        .channels[0]
                        .get_attr_from_name(key),
                        ch_ts.channel_metadata.get_attr_from_name(key),
                    )

        def test_channels(self):
            runs = self.experiment.surveys[0].stations[0].runs
            for run in runs:
                with self.subTest(name=run.id):
                    h5_run = self.mth5_obj.get_run(
                        self.experiment.surveys[0].stations[0].id,
                        run.id,
                        self.survey_name,
                    )
                    for channel in run.channels:
                        h5_channel = h5_run.get_channel(channel.component)

                        sd = channel.to_dict(single=True)
                        sd.pop("hdf5_reference")
                        sd.pop("mth5_type")

                        h5_sd = h5_channel.metadata.to_dict(single=True)
                        h5_sd.pop("hdf5_reference")
                        h5_sd.pop("mth5_type")

                        self.assertDictEqual(h5_sd, sd)

        def test_filters(self):
            exp_filters = self.experiment.surveys[0].filters
            sg = self.mth5_obj.get_survey(self.survey_name)

            for key, value in exp_filters.items():
                with self.subTest(name=key):
                    key = key.replace("/", " per ").lower()
                    sd = value.to_dict(single=True, required=False)
                    h5_sd = sg.filters_group.to_filter_object(key)
                    h5_sd = h5_sd.to_dict(single=True, required=False)
                    for k in sd.keys():
                        with self.subTest(f"{key}_{k}"):
                            v1 = sd[k]
                            v2 = h5_sd[k]
                            if isinstance(v1, (float, int)):
                                self.assertAlmostEqual(v1, float(v2), 5)
                            elif isinstance(v1, np.ndarray):
                                self.assertEqual(v1.dtype, v2.dtype)
                                self.assertTrue((v1 == v2).all())
                            else:
                                self.assertEqual(v1, v2)
                # self.assertDictEqual(h5_sd, sd)

    def test_channel_summary(self):
        self.mth5_obj.channel_summary.summarize()

        with self.subTest("test shape"):
            self.assertEqual(self.mth5_obj.channel_summary.shape, (25,))
        with self.subTest("test nrows"):
            self.assertEqual(self.mth5_obj.channel_summary.nrows, 25)
        with self.subTest(("test dtype")):
            self.assertEqual(self.mth5_obj.channel_summary.dtype, CHANNEL_DTYPE)
        with self.subTest("test station"):
            self.assertTrue(
                (
                    self.mth5_obj.channel_summary.array["station"] == b"REW09"
                ).all()
            )

    @classmethod
    def tearDownClass(self):
        self.mth5_obj.close_mth5()
        self.fn.unlink()


class TestUpdateFromExperiment(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.maxDiff = None
        self.fn = fn_path.joinpath("test.h5")
        self.mth5_obj = mth5.MTH5(file_version="0.1.0")
        self.mth5_obj.open_mth5(self.fn, mode="w")
        self.experiment = Experiment()
        self.experiment.from_xml(fn=MT_EXPERIMENT_SINGLE_STATION)
        self.mth5_obj.from_experiment(self.experiment)

        self.experiment_02 = Experiment()
        self.experiment_02.from_xml(fn=MT_EXPERIMENT_SINGLE_STATION)
        self.experiment_02.surveys[0].id = "different_survey_name"
        self.experiment_02.surveys[0].stations[0].location.latitude = 10

    def test_update_from_new_experiment(self):

        self.mth5_obj.from_experiment(self.experiment_02, update=True)

        with self.subTest("new_survey"):
            self.assertEqual(
                self.mth5_obj.survey_group.metadata.id,
                self.experiment_02.surveys[0].id,
            )
        with self.subTest("new_location"):
            st = self.mth5_obj.get_station("REW09")
            self.assertEqual(
                st.metadata.location.latitude,
                self.experiment_02.surveys[0].stations[0].location.latitude,
            )

    @classmethod
    def tearDownClass(self):
        self.mth5_obj.close_mth5()
        self.fn.unlink()



# -*- coding: utf-8 -*-
"""
Created on Sat Sep 10 21:00:28 2022

@author: jpeacock
"""

# =============================================================================
# Imports
# =============================================================================
import unittest
from pathlib import Path

from mth5.io.zen import Z3DSchedule

# =============================================================================


@unittest.skipIf(
    "peacock" not in str(Path(__file__).as_posix()), "local files"
)
class TestZ3DHeader(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.fn = r"c:\Users\jpeacock\OneDrive - DOI\mt\example_z3d_data\bm100_20220517_131017_256_EY.Z3D"
        self.z3d_obj = Z3DSchedule(fn=self.fn)
        self.z3d_obj.read_schedule()

    def test_AutoGain(self):
        self.assertEqual(getattr(self.z3d_obj, "AutoGain"), "N")

    def test_Comment(self):
        self.assertEqual(getattr(self.z3d_obj, "Comment"), "")

    def test_Date(self):
        self.assertEqual(getattr(self.z3d_obj, "Date"), "2022-05-17")

    def test_Duty(self):
        self.assertEqual(getattr(self.z3d_obj, "Duty"), "0")

    def test_FFTStacks(self):
        self.assertEqual(getattr(self.z3d_obj, "FFTStacks"), "0")

    def test_Filename(self):
        self.assertEqual(getattr(self.z3d_obj, "Filename"), "")

    def test_Gain(self):
        self.assertEqual(getattr(self.z3d_obj, "Gain"), "1.0000")

    def test_Log(self):
        self.assertEqual(getattr(self.z3d_obj, "Log"), "Y")

    def test_NewFile(self):
        self.assertEqual(getattr(self.z3d_obj, "NewFile"), "Y")

    def test_Period(self):
        self.assertEqual(getattr(self.z3d_obj, "Period"), "0")

    def test_RadioOn(self):
        self.assertEqual(getattr(self.z3d_obj, "RadioOn"), "X")

    def test_SR(self):
        self.assertEqual(getattr(self.z3d_obj, "SR"), "256")

    def test_SamplesPerAcq(self):
        self.assertEqual(getattr(self.z3d_obj, "SamplesPerAcq"), "0")

    def test_Sleep(self):
        self.assertEqual(getattr(self.z3d_obj, "Sleep"), "N")

    def test_Sync(self):
        self.assertEqual(getattr(self.z3d_obj, "Sync"), "Y")

    def test_Time(self):
        self.assertEqual(getattr(self.z3d_obj, "Time"), "13:10:15")

    def test_fn(self):
        self.assertEqual(getattr(self.z3d_obj, "fn"), self.fn)

    def test_initial_start(self):
        self.assertEqual(
            getattr(self.z3d_obj, "initial_start"), "2022-05-17T13:09:57+00:00"
        )

    def test_meta_string(self):
        self.assertEqual(
            getattr(self.z3d_obj, "meta_string"),
            b"\n\n\nGPS Brd339/Brd357 Schedule Details\nSchedule.Date = 2022-05-17\nSchedule.Time = 13:10:15\nSchedule.Sync = Y\nSchedule.NewFile = Y\nSchedule.Period = 0\nSchedule.Duty = 0\nSchedule.S/R = 256\nSchedule.Gain = 1.0000\nSchedule.SamplesPerAcq = 0\nSchedule.FFTStacks = 0\nSchedule.Log = Y\nSchedule.Sleep = N\nSchedule.RadioOn = X\nSchedule.AutoGain = N\nSchedule.Filename = \nSchedule.Comment = \n\n\x00                                                                                                                                \r\n\x00",
        )


# =============================================================================
# run
# =============================================================================
if __name__ == "__main__":
    unittest.main()
